"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*!
Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from collections import abc as _abc
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
import builtins as _builtins
import sys
import typing as _typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: _descriptor.FileDescriptor

@_typing.final
class ChatInput(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    class _AcceptFile:
        ValueType = _typing.NewType("ValueType", _builtins.int)
        V: _TypeAlias = ValueType  # noqa: Y015

    class _AcceptFileEnumTypeWrapper(_enum_type_wrapper._EnumTypeWrapper[ChatInput._AcceptFile.ValueType], _builtins.type):
        DESCRIPTOR: _descriptor.EnumDescriptor
        NONE: ChatInput._AcceptFile.ValueType  # 0
        SINGLE: ChatInput._AcceptFile.ValueType  # 1
        MULTIPLE: ChatInput._AcceptFile.ValueType  # 2
        DIRECTORY: ChatInput._AcceptFile.ValueType  # 3

    class AcceptFile(_AcceptFile, metaclass=_AcceptFileEnumTypeWrapper): ...
    NONE: ChatInput.AcceptFile.ValueType  # 0
    SINGLE: ChatInput.AcceptFile.ValueType  # 1
    MULTIPLE: ChatInput.AcceptFile.ValueType  # 2
    DIRECTORY: ChatInput.AcceptFile.ValueType  # 3

    ID_FIELD_NUMBER: _builtins.int
    PLACEHOLDER_FIELD_NUMBER: _builtins.int
    MAX_CHARS_FIELD_NUMBER: _builtins.int
    DISABLED_FIELD_NUMBER: _builtins.int
    VALUE_FIELD_NUMBER: _builtins.int
    SET_VALUE_FIELD_NUMBER: _builtins.int
    DEFAULT_FIELD_NUMBER: _builtins.int
    ACCEPT_FILE_FIELD_NUMBER: _builtins.int
    FILE_TYPE_FIELD_NUMBER: _builtins.int
    MAX_UPLOAD_SIZE_MB_FIELD_NUMBER: _builtins.int
    ACCEPT_AUDIO_FIELD_NUMBER: _builtins.int
    AUDIO_SAMPLE_RATE_FIELD_NUMBER: _builtins.int
    id: _builtins.str
    placeholder: _builtins.str
    max_chars: _builtins.int
    disabled: _builtins.bool
    value: _builtins.str
    set_value: _builtins.bool
    default: _builtins.str
    accept_file: Global___ChatInput.AcceptFile.ValueType
    max_upload_size_mb: _builtins.int
    """Max file size allowed by server config"""
    accept_audio: _builtins.bool
    """Whether to show audio recording button"""
    audio_sample_rate: _builtins.int
    """Sample rate for audio recording in Hz (e.g., 16000, 48000)
    If not set, uses browser default
    """
    @_builtins.property
    def file_type(self) -> _containers.RepeatedScalarFieldContainer[_builtins.str]:
        """Supported file types: For example: ["png","jpg","img"]"""

    def __init__(
        self,
        *,
        id: _builtins.str = ...,
        placeholder: _builtins.str = ...,
        max_chars: _builtins.int = ...,
        disabled: _builtins.bool = ...,
        value: _builtins.str = ...,
        set_value: _builtins.bool = ...,
        default: _builtins.str = ...,
        accept_file: Global___ChatInput.AcceptFile.ValueType = ...,
        file_type: _abc.Iterable[_builtins.str] | None = ...,
        max_upload_size_mb: _builtins.int = ...,
        accept_audio: _builtins.bool = ...,
        audio_sample_rate: _builtins.int | None = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["_audio_sample_rate", b"_audio_sample_rate", "audio_sample_rate", b"audio_sample_rate"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["_audio_sample_rate", b"_audio_sample_rate", "accept_audio", b"accept_audio", "accept_file", b"accept_file", "audio_sample_rate", b"audio_sample_rate", "default", b"default", "disabled", b"disabled", "file_type", b"file_type", "id", b"id", "max_chars", b"max_chars", "max_upload_size_mb", b"max_upload_size_mb", "placeholder", b"placeholder", "set_value", b"set_value", "value", b"value"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType__audio_sample_rate: _TypeAlias = _typing.Literal["audio_sample_rate"]  # noqa: Y015
    _WhichOneofArgType__audio_sample_rate: _TypeAlias = _typing.Literal["_audio_sample_rate", b"_audio_sample_rate"]  # noqa: Y015
    def WhichOneof(self, oneof_group: _WhichOneofArgType__audio_sample_rate) -> _WhichOneofReturnType__audio_sample_rate | None: ...

Global___ChatInput: _TypeAlias = ChatInput  # noqa: Y015



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*!
Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from collections import abc as _abc
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
import builtins as _builtins
import sys
import typing as _typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: _descriptor.FileDescriptor

@_typing.final
class PlotlyChart(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    class _SelectionMode:
        ValueType = _typing.NewType("ValueType", _builtins.int)
        V: _TypeAlias = ValueType  # noqa: Y015

    class _SelectionModeEnumTypeWrapper(_enum_type_wrapper._EnumTypeWrapper[PlotlyChart._SelectionMode.ValueType], _builtins.type):
        DESCRIPTOR: _descriptor.EnumDescriptor
        POINTS: PlotlyChart._SelectionMode.ValueType  # 0
        """Point selection mode"""
        BOX: PlotlyChart._SelectionMode.ValueType  # 1
        """Box selection mode"""
        LASSO: PlotlyChart._SelectionMode.ValueType  # 2
        """Lasso selection mode"""

    class SelectionMode(_SelectionMode, metaclass=_SelectionModeEnumTypeWrapper):
        """Available selection modes:"""

    POINTS: PlotlyChart.SelectionMode.ValueType  # 0
    """Point selection mode"""
    BOX: PlotlyChart.SelectionMode.ValueType  # 1
    """Box selection mode"""
    LASSO: PlotlyChart.SelectionMode.ValueType  # 2
    """Lasso selection mode"""

    THEME_FIELD_NUMBER: _builtins.int
    ID_FIELD_NUMBER: _builtins.int
    SELECTION_MODE_FIELD_NUMBER: _builtins.int
    FORM_ID_FIELD_NUMBER: _builtins.int
    SPEC_FIELD_NUMBER: _builtins.int
    CONFIG_FIELD_NUMBER: _builtins.int
    theme: _builtins.str
    """override the properties with a theme. Currently, only "streamlit" or None are accepted."""
    id: _builtins.str
    """The unique element ID of this chart."""
    form_id: _builtins.str
    """Form ID, filled if selections are activated."""
    spec: _builtins.str
    """JSON-serialized dict containing keys from the set {data, frames, layout}."""
    config: _builtins.str
    """JSON-serialized dict with Plotly's config object."""
    @_builtins.property
    def selection_mode(self) -> _containers.RepeatedScalarFieldContainer[Global___PlotlyChart.SelectionMode.ValueType]:
        """Activate selections types on the chart."""

    def __init__(
        self,
        *,
        theme: _builtins.str = ...,
        id: _builtins.str = ...,
        selection_mode: _abc.Iterable[Global___PlotlyChart.SelectionMode.ValueType] | None = ...,
        form_id: _builtins.str = ...,
        spec: _builtins.str = ...,
        config: _builtins.str = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["config", b"config", "form_id", b"form_id", "id", b"id", "selection_mode", b"selection_mode", "spec", b"spec", "theme", b"theme"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___PlotlyChart: _TypeAlias = PlotlyChart  # noqa: Y015



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*!
Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from collections import abc as _abc
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
import builtins as _builtins
import sys
import typing as _typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: _descriptor.FileDescriptor

@_typing.final
class GitInfo(_message.Message):
    """Message used to update page metadata."""

    DESCRIPTOR: _descriptor.Descriptor

    class _GitStates:
        ValueType = _typing.NewType("ValueType", _builtins.int)
        V: _TypeAlias = ValueType  # noqa: Y015

    class _GitStatesEnumTypeWrapper(_enum_type_wrapper._EnumTypeWrapper[GitInfo._GitStates.ValueType], _builtins.type):
        DESCRIPTOR: _descriptor.EnumDescriptor
        DEFAULT: GitInfo._GitStates.ValueType  # 0
        HEAD_DETACHED: GitInfo._GitStates.ValueType  # 1
        AHEAD_OF_REMOTE: GitInfo._GitStates.ValueType  # 2

    class GitStates(_GitStates, metaclass=_GitStatesEnumTypeWrapper): ...
    DEFAULT: GitInfo.GitStates.ValueType  # 0
    HEAD_DETACHED: GitInfo.GitStates.ValueType  # 1
    AHEAD_OF_REMOTE: GitInfo.GitStates.ValueType  # 2

    REPOSITORY_FIELD_NUMBER: _builtins.int
    BRANCH_FIELD_NUMBER: _builtins.int
    MODULE_FIELD_NUMBER: _builtins.int
    UNTRACKED_FILES_FIELD_NUMBER: _builtins.int
    UNCOMMITTED_FILES_FIELD_NUMBER: _builtins.int
    STATE_FIELD_NUMBER: _builtins.int
    repository: _builtins.str
    branch: _builtins.str
    module: _builtins.str
    state: Global___GitInfo.GitStates.ValueType
    @_builtins.property
    def untracked_files(self) -> _containers.RepeatedScalarFieldContainer[_builtins.str]: ...
    @_builtins.property
    def uncommitted_files(self) -> _containers.RepeatedScalarFieldContainer[_builtins.str]: ...
    def __init__(
        self,
        *,
        repository: _builtins.str = ...,
        branch: _builtins.str = ...,
        module: _builtins.str = ...,
        untracked_files: _abc.Iterable[_builtins.str] | None = ...,
        uncommitted_files: _abc.Iterable[_builtins.str] | None = ...,
        state: Global___GitInfo.GitStates.ValueType = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["branch", b"branch", "module", b"module", "repository", b"repository", "state", b"state", "uncommitted_files", b"uncommitted_files", "untracked_files", b"untracked_files"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___GitInfo: _TypeAlias = GitInfo  # noqa: Y015



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*!
Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from collections import abc as _abc
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf.internal import containers as _containers
from streamlit.proto import LabelVisibility_pb2 as _LabelVisibility_pb2
import builtins as _builtins
import sys
import typing as _typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: _descriptor.FileDescriptor

@_typing.final
class MultiSelect(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    ID_FIELD_NUMBER: _builtins.int
    LABEL_FIELD_NUMBER: _builtins.int
    DEFAULT_FIELD_NUMBER: _builtins.int
    OPTIONS_FIELD_NUMBER: _builtins.int
    HELP_FIELD_NUMBER: _builtins.int
    FORM_ID_FIELD_NUMBER: _builtins.int
    RAW_VALUES_FIELD_NUMBER: _builtins.int
    SET_VALUE_FIELD_NUMBER: _builtins.int
    DISABLED_FIELD_NUMBER: _builtins.int
    LABEL_VISIBILITY_FIELD_NUMBER: _builtins.int
    MAX_SELECTIONS_FIELD_NUMBER: _builtins.int
    PLACEHOLDER_FIELD_NUMBER: _builtins.int
    ACCEPT_NEW_OPTIONS_FIELD_NUMBER: _builtins.int
    QUERY_PARAM_KEY_FIELD_NUMBER: _builtins.int
    id: _builtins.str
    label: _builtins.str
    help: _builtins.str
    form_id: _builtins.str
    set_value: _builtins.bool
    disabled: _builtins.bool
    max_selections: _builtins.int
    placeholder: _builtins.str
    accept_new_options: _builtins.bool
    query_param_key: _builtins.str
    """If set, widget value is bound to this query parameter key"""
    @_builtins.property
    def default(self) -> _containers.RepeatedScalarFieldContainer[_builtins.int]: ...
    @_builtins.property
    def options(self) -> _containers.RepeatedScalarFieldContainer[_builtins.str]: ...
    @_builtins.property
    def raw_values(self) -> _containers.RepeatedScalarFieldContainer[_builtins.str]:
        """after we introduced accept_new_options, we send the option as a string
        instead of an index to keep it simple.
        """

    @_builtins.property
    def label_visibility(self) -> _LabelVisibility_pb2.LabelVisibility: ...
    def __init__(
        self,
        *,
        id: _builtins.str = ...,
        label: _builtins.str = ...,
        default: _abc.Iterable[_builtins.int] | None = ...,
        options: _abc.Iterable[_builtins.str] | None = ...,
        help: _builtins.str = ...,
        form_id: _builtins.str = ...,
        raw_values: _abc.Iterable[_builtins.str] | None = ...,
        set_value: _builtins.bool = ...,
        disabled: _builtins.bool = ...,
        label_visibility: _LabelVisibility_pb2.LabelVisibility | None = ...,
        max_selections: _builtins.int = ...,
        placeholder: _builtins.str = ...,
        accept_new_options: _builtins.bool | None = ...,
        query_param_key: _builtins.str | None = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["_accept_new_options", b"_accept_new_options", "_query_param_key", b"_query_param_key", "accept_new_options", b"accept_new_options", "label_visibility", b"label_visibility", "query_param_key", b"query_param_key"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["_accept_new_options", b"_accept_new_options", "_query_param_key", b"_query_param_key", "accept_new_options", b"accept_new_options", "default", b"default", "disabled", b"disabled", "form_id", b"form_id", "help", b"help", "id", b"id", "label", b"label", "label_visibility", b"label_visibility", "max_selections", b"max_selections", "options", b"options", "placeholder", b"placeholder", "query_param_key", b"query_param_key", "raw_values", b"raw_values", "set_value", b"set_value"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType__accept_new_options: _TypeAlias = _typing.Literal["accept_new_options"]  # noqa: Y015
    _WhichOneofArgType__accept_new_options: _TypeAlias = _typing.Literal["_accept_new_options", b"_accept_new_options"]  # noqa: Y015
    _WhichOneofReturnType__query_param_key: _TypeAlias = _typing.Literal["query_param_key"]  # noqa: Y015
    _WhichOneofArgType__query_param_key: _TypeAlias = _typing.Literal["_query_param_key", b"_query_param_key"]  # noqa: Y015
    @_typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__accept_new_options) -> _WhichOneofReturnType__accept_new_options | None: ...
    @_typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__query_param_key) -> _WhichOneofReturnType__query_param_key | None: ...

Global___MultiSelect: _TypeAlias = MultiSelect  # noqa: Y015



# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Utility functions for the Starlette server implementation."""

from __future__ import annotations

import binascii
import os
import time


def parse_range_header(range_header: str, total_size: int) -> tuple[int, int]:
    """Parse the Range header and return the start and end byte positions.

    This is used for serving media files with range requests.

    Parameters
    ----------
    range_header : str
        The value of the Range header (e.g. "bytes=0-1023").

    total_size : int
        The total size of the resource in bytes.

    Returns
    -------
    tuple[int, int]
        A tuple containing (start, end) byte positions.
    """
    if total_size <= 0:
        raise ValueError("empty content")

    units, sep, range_spec = range_header.partition("=")
    if units.strip().lower() != "bytes" or sep == "" or "," in range_spec:
        raise ValueError("invalid range")

    range_spec = range_spec.strip()
    if range_spec.startswith("-"):
        try:
            suffix = int(range_spec[1:])
        except ValueError:
            raise ValueError("invalid suffix range") from None
        if suffix <= 0:
            raise ValueError("invalid suffix range")
        if suffix >= total_size:
            return 0, total_size - 1
        return total_size - suffix, total_size - 1

    start_str, sep, end_str = range_spec.partition("-")
    if not start_str:
        raise ValueError("missing range start")

    start = int(start_str)
    if start < 0 or start >= total_size:
        raise ValueError("start out of range")

    if sep == "" or not end_str:
        end = total_size - 1
    else:
        end = int(end_str)
        if end < start:
            raise ValueError("end before start")
        end = min(end, total_size - 1)

    return start, end


def websocket_mask(mask: bytes, data: bytes) -> bytes:
    """Mask or unmask data for WebSocket transmission per RFC 6455.

    Each byte of data is XORed with mask[i % 4]. This operation is
    bidirectional - applying it twice with the same mask returns the
    original data.

    Parameters
    ----------
    mask : bytes
        A 4-byte masking key.
    data : bytes
        The data to mask or unmask.

    Returns
    -------
    bytes
        The masked/unmasked data.
    """
    if len(mask) != 4:
        raise ValueError("mask must be 4 bytes")

    result = bytearray(len(data))
    for i, byte in enumerate(data):
        result[i] = byte ^ mask[i % 4]
    return bytes(result)


def create_signed_value(
    secret: str,
    name: str,
    value: str | bytes,
) -> bytes:
    """Create a signed cookie value using itsdangerous.

    Note: This uses itsdangerous for signing, which is NOT compatible with
    Tornado's secure cookie format. Cookies signed by this function cannot
    be read by Tornado's get_signed_cookie/get_secure_cookie, and vice versa.
    Switching between Tornado and Starlette backends will invalidate existing
    auth cookies (_streamlit_user), requiring users to re-authenticate.
    This is expected behavior when switching between Tornado and Starlette backends.

    Parameters
    ----------
    secret
        The secret key used for signing.
    name
        The cookie name (used as salt for additional security).
    value
        The value to sign.

    Returns
    -------
    bytes
        The signed value as bytes.
    """
    from itsdangerous import URLSafeTimedSerializer

    serializer = URLSafeTimedSerializer(secret, salt=name)
    if isinstance(value, bytes):
        value = value.decode("utf-8")
    return serializer.dumps(value).encode("utf-8")


def decode_signed_value(
    secret: str,
    name: str,
    value: str | bytes,
    max_age_days: float = 31,
) -> bytes | None:
    """Decode a signed cookie value using itsdangerous.

    Parameters
    ----------
    secret
        The secret key used for signing.
    name
        The cookie name (used as salt for additional security).
    value
        The signed value to decode.
    max_age_days
        Maximum age of the cookie in days (default: 31).

    Returns
    -------
    bytes | None
        The decoded value as bytes, or None if invalid/expired.
    """
    from itsdangerous import BadSignature, SignatureExpired, URLSafeTimedSerializer

    if not value:
        return None

    try:
        if isinstance(value, bytes):
            value = value.decode("utf-8")

        serializer = URLSafeTimedSerializer(secret, salt=name)
        decoded = serializer.loads(value, max_age=int(max_age_days * 86400))
        if isinstance(decoded, str):
            return decoded.encode("utf-8")
        if isinstance(decoded, bytes):
            return decoded
        # Unexpected type from deserializer — treat as invalid
        return None
    except (BadSignature, SignatureExpired, UnicodeDecodeError):
        return None


def generate_xsrf_token_string(
    token_bytes: bytes | None = None, timestamp: int | None = None
) -> str:
    """Generate a version 2 XSRF token string compatible with Tornado.

    Format: 2|mask|masked_token|timestamp

    Parameters
    ----------
    token_bytes
        The raw token bytes to encode. If None, generates 16 random bytes.
    timestamp
        The Unix timestamp to include in the token. If None, uses current time.

    Returns
    -------
    str
        The encoded XSRF token string in version 2 format.
    """
    if token_bytes is None:
        token_bytes = os.urandom(16)
    if timestamp is None:
        timestamp = int(time.time())

    mask = os.urandom(4)
    masked_token = websocket_mask(mask, token_bytes)
    return "2|{}|{}|{}".format(
        binascii.b2a_hex(mask).decode("ascii"),
        binascii.b2a_hex(masked_token).decode("ascii"),
        timestamp,
    )


def decode_xsrf_token_string(
    cookie_value: str,
) -> tuple[bytes | None, int | None]:
    """Decode a Tornado XSRF token string.

    Supports version 2 (masked) and version 1 (unmasked) tokens.

    Parameters
    ----------
    cookie_value
        The XSRF token cookie value to decode.

    Returns
    -------
    tuple[bytes | None, int | None]
        A tuple of (token_bytes, timestamp). Both values are None if decoding fails.
    """
    if not cookie_value:
        return None, None

    value = cookie_value.strip("\"'")
    if not value:
        return None, None

    try:
        # V2 tokens:
        if value.startswith("2|"):
            _, mask_hex, masked_hex, timestamp_str = value.split("|")
            mask = binascii.a2b_hex(mask_hex.encode("ascii"))
            masked = binascii.a2b_hex(masked_hex.encode("ascii"))
            token = websocket_mask(mask, masked)
            return token, int(timestamp_str)

        # V1 tokens:
        # TODO(lukasmasuch): This is likely unused in Streamlit since only V2 tokens
        # are used. We might be able to just remove this part.
        token = binascii.a2b_hex(value.encode("ascii"))
        if not token:
            return None, None
        # V1 tokens don't have an embedded timestamp, so we use current time
        # as a placeholder (matches Tornado's behavior). This timestamp is
        # informational only and not used for token validation.
        return token, int(time.time())
    except (binascii.Error, ValueError):
        return None, None


def generate_random_hex_string(num_bytes: int = 32) -> str:
    """Generate a cryptographically secure random hex string.

    Parameters
    ----------
    num_bytes
        Number of random bytes to generate (default: 32).
        The resulting hex string will be twice this length.

    Returns
    -------
    str
        A hex-encoded random string.
    """
    return binascii.b2a_hex(os.urandom(num_bytes)).decode("ascii")


def validate_xsrf_token(supplied_token: str | None, xsrf_cookie: str | None) -> bool:
    """Validate the XSRF token from the WebSocket subprotocol against the cookie.

    This mirrors Tornado's XSRF validation logic to ensure the frontend can share
    XSRF logic between WebSocket handshake and HTTP uploads regardless of backend.
    """

    if not supplied_token or not xsrf_cookie:
        return False

    # Decode the supplied token from the subprotocol
    supplied_token_bytes, _ = decode_xsrf_token_string(supplied_token)
    # Decode the expected token from the cookie
    expected_token_bytes, _ = decode_xsrf_token_string(xsrf_cookie)

    if not supplied_token_bytes or not expected_token_bytes:
        return False

    import hmac

    return hmac.compare_digest(supplied_token_bytes, expected_token_bytes)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*!
Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from collections import abc as _abc
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from streamlit.proto import AppPage_pb2 as _AppPage_pb2
import builtins as _builtins
import sys
import typing as _typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: _descriptor.FileDescriptor

@_typing.final
class Navigation(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    class _Position:
        ValueType = _typing.NewType("ValueType", _builtins.int)
        V: _TypeAlias = ValueType  # noqa: Y015

    class _PositionEnumTypeWrapper(_enum_type_wrapper._EnumTypeWrapper[Navigation._Position.ValueType], _builtins.type):
        DESCRIPTOR: _descriptor.EnumDescriptor
        HIDDEN: Navigation._Position.ValueType  # 0
        """do not display the navigation"""
        SIDEBAR: Navigation._Position.ValueType  # 1
        """display navigation in the sidebar"""
        TOP: Navigation._Position.ValueType  # 2
        """display navigation in the top header"""

    class Position(_Position, metaclass=_PositionEnumTypeWrapper):
        """Position of the Navigation"""

    HIDDEN: Navigation.Position.ValueType  # 0
    """do not display the navigation"""
    SIDEBAR: Navigation.Position.ValueType  # 1
    """display navigation in the sidebar"""
    TOP: Navigation.Position.ValueType  # 2
    """display navigation in the top header"""

    SECTIONS_FIELD_NUMBER: _builtins.int
    APP_PAGES_FIELD_NUMBER: _builtins.int
    POSITION_FIELD_NUMBER: _builtins.int
    PAGE_SCRIPT_HASH_FIELD_NUMBER: _builtins.int
    EXPANDED_FIELD_NUMBER: _builtins.int
    VISIBLE_ITEMS_FIELD_NUMBER: _builtins.int
    position: Global___Navigation.Position.ValueType
    page_script_hash: _builtins.str
    """The script hash for the page identified by st.navigation"""
    expanded: _builtins.bool
    visible_items: _builtins.int
    """Maximum number of pages to display when the navigation is collapsed.
    When not set, uses the default (10 pages).
    When set to a positive value, shows that many pages before "View X more".
    """
    @_builtins.property
    def sections(self) -> _containers.RepeatedScalarFieldContainer[_builtins.str]: ...
    @_builtins.property
    def app_pages(self) -> _containers.RepeatedCompositeFieldContainer[_AppPage_pb2.AppPage]: ...
    def __init__(
        self,
        *,
        sections: _abc.Iterable[_builtins.str] | None = ...,
        app_pages: _abc.Iterable[_AppPage_pb2.AppPage] | None = ...,
        position: Global___Navigation.Position.ValueType = ...,
        page_script_hash: _builtins.str = ...,
        expanded: _builtins.bool = ...,
        visible_items: _builtins.int | None = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["_visible_items", b"_visible_items", "visible_items", b"visible_items"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["_visible_items", b"_visible_items", "app_pages", b"app_pages", "expanded", b"expanded", "page_script_hash", b"page_script_hash", "position", b"position", "sections", b"sections", "visible_items", b"visible_items"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType__visible_items: _TypeAlias = _typing.Literal["visible_items"]  # noqa: Y015
    _WhichOneofArgType__visible_items: _TypeAlias = _typing.Literal["_visible_items", b"_visible_items"]  # noqa: Y015
    def WhichOneof(self, oneof_group: _WhichOneofArgType__visible_items) -> _WhichOneofReturnType__visible_items | None: ...

Global___Navigation: _TypeAlias = Navigation  # noqa: Y015



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*!
Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from streamlit.proto import GapSize_pb2 as _GapSize_pb2
from streamlit.proto import HeightConfig_pb2 as _HeightConfig_pb2
from streamlit.proto import WidthConfig_pb2 as _WidthConfig_pb2
import builtins as _builtins
import sys
import typing as _typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: _descriptor.FileDescriptor

@_typing.final
class Block(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    @_typing.final
    class Vertical(_message.Message):
        DESCRIPTOR: _descriptor.Descriptor

        BORDER_FIELD_NUMBER: _builtins.int
        border: _builtins.bool
        def __init__(
            self,
            *,
            border: _builtins.bool = ...,
        ) -> None: ...
        _ClearFieldArgType: _TypeAlias = _typing.Literal["border", b"border"]  # noqa: Y015
        def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

    @_typing.final
    class Horizontal(_message.Message):
        DESCRIPTOR: _descriptor.Descriptor

        GAP_FIELD_NUMBER: _builtins.int
        gap: _builtins.str
        def __init__(
            self,
            *,
            gap: _builtins.str = ...,
        ) -> None: ...
        _ClearFieldArgType: _TypeAlias = _typing.Literal["gap", b"gap"]  # noqa: Y015
        def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

    @_typing.final
    class FlexContainer(_message.Message):
        DESCRIPTOR: _descriptor.Descriptor

        class _Direction:
            ValueType = _typing.NewType("ValueType", _builtins.int)
            V: _TypeAlias = ValueType  # noqa: Y015

        class _DirectionEnumTypeWrapper(_enum_type_wrapper._EnumTypeWrapper[Block.FlexContainer._Direction.ValueType], _builtins.type):
            DESCRIPTOR: _descriptor.EnumDescriptor
            DIRECTION_UNDEFINED: Block.FlexContainer._Direction.ValueType  # 0
            VERTICAL: Block.FlexContainer._Direction.ValueType  # 1
            HORIZONTAL: Block.FlexContainer._Direction.ValueType  # 2

        class Direction(_Direction, metaclass=_DirectionEnumTypeWrapper): ...
        DIRECTION_UNDEFINED: Block.FlexContainer.Direction.ValueType  # 0
        VERTICAL: Block.FlexContainer.Direction.ValueType  # 1
        HORIZONTAL: Block.FlexContainer.Direction.ValueType  # 2

        class _Justify:
            ValueType = _typing.NewType("ValueType", _builtins.int)
            V: _TypeAlias = ValueType  # noqa: Y015

        class _JustifyEnumTypeWrapper(_enum_type_wrapper._EnumTypeWrapper[Block.FlexContainer._Justify.ValueType], _builtins.type):
            DESCRIPTOR: _descriptor.EnumDescriptor
            JUSTIFY_UNDEFINED: Block.FlexContainer._Justify.ValueType  # 0
            JUSTIFY_START: Block.FlexContainer._Justify.ValueType  # 1
            JUSTIFY_END: Block.FlexContainer._Justify.ValueType  # 2
            JUSTIFY_CENTER: Block.FlexContainer._Justify.ValueType  # 3
            SPACE_BETWEEN: Block.FlexContainer._Justify.ValueType  # 4

        class Justify(_Justify, metaclass=_JustifyEnumTypeWrapper): ...
        JUSTIFY_UNDEFINED: Block.FlexContainer.Justify.ValueType  # 0
        JUSTIFY_START: Block.FlexContainer.Justify.ValueType  # 1
        JUSTIFY_END: Block.FlexContainer.Justify.ValueType  # 2
        JUSTIFY_CENTER: Block.FlexContainer.Justify.ValueType  # 3
        SPACE_BETWEEN: Block.FlexContainer.Justify.ValueType  # 4

        class _Align:
            ValueType = _typing.NewType("ValueType", _builtins.int)
            V: _TypeAlias = ValueType  # noqa: Y015

        class _AlignEnumTypeWrapper(_enum_type_wrapper._EnumTypeWrapper[Block.FlexContainer._Align.ValueType], _builtins.type):
            DESCRIPTOR: _descriptor.EnumDescriptor
            ALIGN_UNDEFINED: Block.FlexContainer._Align.ValueType  # 0
            ALIGN_START: Block.FlexContainer._Align.ValueType  # 1
            ALIGN_END: Block.FlexContainer._Align.ValueType  # 2
            ALIGN_CENTER: Block.FlexContainer._Align.ValueType  # 3
            STRETCH: Block.FlexContainer._Align.ValueType  # 4

        class Align(_Align, metaclass=_AlignEnumTypeWrapper): ...
        ALIGN_UNDEFINED: Block.FlexContainer.Align.ValueType  # 0
        ALIGN_START: Block.FlexContainer.Align.ValueType  # 1
        ALIGN_END: Block.FlexContainer.Align.ValueType  # 2
        ALIGN_CENTER: Block.FlexContainer.Align.ValueType  # 3
        STRETCH: Block.FlexContainer.Align.ValueType  # 4

        BORDER_FIELD_NUMBER: _builtins.int
        GAP_CONFIG_FIELD_NUMBER: _builtins.int
        SCALE_FIELD_NUMBER: _builtins.int
        DIRECTION_FIELD_NUMBER: _builtins.int
        WRAP_FIELD_NUMBER: _builtins.int
        JUSTIFY_FIELD_NUMBER: _builtins.int
        ALIGN_FIELD_NUMBER: _builtins.int
        border: _builtins.bool
        scale: _builtins.float
        direction: Global___Block.FlexContainer.Direction.ValueType
        wrap: _builtins.bool
        justify: Global___Block.FlexContainer.Justify.ValueType
        align: Global___Block.FlexContainer.Align.ValueType
        @_builtins.property
        def gap_config(self) -> _GapSize_pb2.GapConfig: ...
        def __init__(
            self,
            *,
            border: _builtins.bool = ...,
            gap_config: _GapSize_pb2.GapConfig | None = ...,
            scale: _builtins.float = ...,
            direction: Global___Block.FlexContainer.Direction.ValueType = ...,
            wrap: _builtins.bool = ...,
            justify: Global___Block.FlexContainer.Justify.ValueType = ...,
            align: Global___Block.FlexContainer.Align.ValueType = ...,
        ) -> None: ...
        _HasFieldArgType: _TypeAlias = _typing.Literal["gap_config", b"gap_config"]  # noqa: Y015
        def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
        _ClearFieldArgType: _TypeAlias = _typing.Literal["align", b"align", "border", b"border", "direction", b"direction", "gap_config", b"gap_config", "justify", b"justify", "scale", b"scale", "wrap", b"wrap"]  # noqa: Y015
        def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

    @_typing.final
    class Column(_message.Message):
        DESCRIPTOR: _descriptor.Descriptor

        class _VerticalAlignment:
            ValueType = _typing.NewType("ValueType", _builtins.int)
            V: _TypeAlias = ValueType  # noqa: Y015

        class _VerticalAlignmentEnumTypeWrapper(_enum_type_wrapper._EnumTypeWrapper[Block.Column._VerticalAlignment.ValueType], _builtins.type):
            DESCRIPTOR: _descriptor.EnumDescriptor
            TOP: Block.Column._VerticalAlignment.ValueType  # 0
            CENTER: Block.Column._VerticalAlignment.ValueType  # 1
            BOTTOM: Block.Column._VerticalAlignment.ValueType  # 2

        class VerticalAlignment(_VerticalAlignment, metaclass=_VerticalAlignmentEnumTypeWrapper): ...
        TOP: Block.Column.VerticalAlignment.ValueType  # 0
        CENTER: Block.Column.VerticalAlignment.ValueType  # 1
        BOTTOM: Block.Column.VerticalAlignment.ValueType  # 2

        WEIGHT_FIELD_NUMBER: _builtins.int
        VERTICAL_ALIGNMENT_FIELD_NUMBER: _builtins.int
        SHOW_BORDER_FIELD_NUMBER: _builtins.int
        GAP_CONFIG_FIELD_NUMBER: _builtins.int
        weight: _builtins.float
        vertical_alignment: Global___Block.Column.VerticalAlignment.ValueType
        show_border: _builtins.bool
        @_builtins.property
        def gap_config(self) -> _GapSize_pb2.GapConfig: ...
        def __init__(
            self,
            *,
            weight: _builtins.float = ...,
            vertical_alignment: Global___Block.Column.VerticalAlignment.ValueType = ...,
            show_border: _builtins.bool = ...,
            gap_config: _GapSize_pb2.GapConfig | None = ...,
        ) -> None: ...
        _HasFieldArgType: _TypeAlias = _typing.Literal["_gap_config", b"_gap_config", "gap_config", b"gap_config"]  # noqa: Y015
        def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
        _ClearFieldArgType: _TypeAlias = _typing.Literal["_gap_config", b"_gap_config", "gap_config", b"gap_config", "show_border", b"show_border", "vertical_alignment", b"vertical_alignment", "weight", b"weight"]  # noqa: Y015
        def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
        _WhichOneofReturnType__gap_config: _TypeAlias = _typing.Literal["gap_config"]  # noqa: Y015
        _WhichOneofArgType__gap_config: _TypeAlias = _typing.Literal["_gap_config", b"_gap_config"]  # noqa: Y015
        def WhichOneof(self, oneof_group: _WhichOneofArgType__gap_config) -> _WhichOneofReturnType__gap_config | None: ...

    @_typing.final
    class Expandable(_message.Message):
        DESCRIPTOR: _descriptor.Descriptor

        LABEL_FIELD_NUMBER: _builtins.int
        EXPANDED_FIELD_NUMBER: _builtins.int
        ICON_FIELD_NUMBER: _builtins.int
        ID_FIELD_NUMBER: _builtins.int
        label: _builtins.str
        expanded: _builtins.bool
        icon: _builtins.str
        id: _builtins.str
        """ID for dynamic expanders. Only set when on_change="rerun",
        signaling the frontend to treat this as a stateful widget.
        """
        def __init__(
            self,
            *,
            label: _builtins.str = ...,
            expanded: _builtins.bool | None = ...,
            icon: _builtins.str = ...,
            id: _builtins.str | None = ...,
        ) -> None: ...
        _HasFieldArgType: _TypeAlias = _typing.Literal["_expanded", b"_expanded", "_id", b"_id", "expanded", b"expanded", "id", b"id"]  # noqa: Y015
        def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
        _ClearFieldArgType: _TypeAlias = _typing.Literal["_expanded", b"_expanded", "_id", b"_id", "expanded", b"expanded", "icon", b"icon", "id", b"id", "label", b"label"]  # noqa: Y015
        def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
        _WhichOneofReturnType__expanded: _TypeAlias = _typing.Literal["expanded"]  # noqa: Y015
        _WhichOneofArgType__expanded: _TypeAlias = _typing.Literal["_expanded", b"_expanded"]  # noqa: Y015
        _WhichOneofReturnType__id: _TypeAlias = _typing.Literal["id"]  # noqa: Y015
        _WhichOneofArgType__id: _TypeAlias = _typing.Literal["_id", b"_id"]  # noqa: Y015
        @_typing.overload
        def WhichOneof(self, oneof_group: _WhichOneofArgType__expanded) -> _WhichOneofReturnType__expanded | None: ...
        @_typing.overload
        def WhichOneof(self, oneof_group: _WhichOneofArgType__id) -> _WhichOneofReturnType__id | None: ...

    @_typing.final
    class Dialog(_message.Message):
        DESCRIPTOR: _descriptor.Descriptor

        class _DialogWidth:
            ValueType = _typing.NewType("ValueType", _builtins.int)
            V: _TypeAlias = ValueType  # noqa: Y015

        class _DialogWidthEnumTypeWrapper(_enum_type_wrapper._EnumTypeWrapper[Block.Dialog._DialogWidth.ValueType], _builtins.type):
            DESCRIPTOR: _descriptor.EnumDescriptor
            SMALL: Block.Dialog._DialogWidth.ValueType  # 0
            LARGE: Block.Dialog._DialogWidth.ValueType  # 1
            MEDIUM: Block.Dialog._DialogWidth.ValueType  # 2

        class DialogWidth(_DialogWidth, metaclass=_DialogWidthEnumTypeWrapper): ...
        SMALL: Block.Dialog.DialogWidth.ValueType  # 0
        LARGE: Block.Dialog.DialogWidth.ValueType  # 1
        MEDIUM: Block.Dialog.DialogWidth.ValueType  # 2

        TITLE_FIELD_NUMBER: _builtins.int
        DISMISSIBLE_FIELD_NUMBER: _builtins.int
        WIDTH_FIELD_NUMBER: _builtins.int
        IS_OPEN_FIELD_NUMBER: _builtins.int
        ID_FIELD_NUMBER: _builtins.int
        ICON_FIELD_NUMBER: _builtins.int
        title: _builtins.str
        dismissible: _builtins.bool
        width: Global___Block.Dialog.DialogWidth.ValueType
        is_open: _builtins.bool
        id: _builtins.str
        """If id is set, dismissal of the dialog will trigger a rerun."""
        icon: _builtins.str
        """Optional icon shown next to the dialog title."""
        def __init__(
            self,
            *,
            title: _builtins.str = ...,
            dismissible: _builtins.bool = ...,
            width: Global___Block.Dialog.DialogWidth.ValueType = ...,
            is_open: _builtins.bool | None = ...,
            id: _builtins.str = ...,
            icon: _builtins.str = ...,
        ) -> None: ...
        _HasFieldArgType: _TypeAlias = _typing.Literal["_is_open", b"_is_open", "is_open", b"is_open"]  # noqa: Y015
        def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
        _ClearFieldArgType: _TypeAlias = _typing.Literal["_is_open", b"_is_open", "dismissible", b"dismissible", "icon", b"icon", "id", b"id", "is_open", b"is_open", "title", b"title", "width", b"width"]  # noqa: Y015
        def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
        _WhichOneofReturnType__is_open: _TypeAlias = _typing.Literal["is_open"]  # noqa: Y015
        _WhichOneofArgType__is_open: _TypeAlias = _typing.Literal["_is_open", b"_is_open"]  # noqa: Y015
        def WhichOneof(self, oneof_group: _WhichOneofArgType__is_open) -> _WhichOneofReturnType__is_open | None: ...

    @_typing.final
    class Form(_message.Message):
        DESCRIPTOR: _descriptor.Descriptor

        FORM_ID_FIELD_NUMBER: _builtins.int
        CLEAR_ON_SUBMIT_FIELD_NUMBER: _builtins.int
        BORDER_FIELD_NUMBER: _builtins.int
        ENTER_TO_SUBMIT_FIELD_NUMBER: _builtins.int
        form_id: _builtins.str
        clear_on_submit: _builtins.bool
        border: _builtins.bool
        enter_to_submit: _builtins.bool
        def __init__(
            self,
            *,
            form_id: _builtins.str = ...,
            clear_on_submit: _builtins.bool = ...,
            border: _builtins.bool = ...,
            enter_to_submit: _builtins.bool = ...,
        ) -> None: ...
        _ClearFieldArgType: _TypeAlias = _typing.Literal["border", b"border", "clear_on_submit", b"clear_on_submit", "enter_to_submit", b"enter_to_submit", "form_id", b"form_id"]  # noqa: Y015
        def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

    @_typing.final
    class TabContainer(_message.Message):
        DESCRIPTOR: _descriptor.Descriptor

        DEFAULT_TAB_INDEX_FIELD_NUMBER: _builtins.int
        ID_FIELD_NUMBER: _builtins.int
        default_tab_index: _builtins.int
        id: _builtins.str
        """Widget ID for dynamic tabs. Only set when on_change="rerun",
        signaling the frontend to treat this as a stateful widget.
        When absent, tabs may still have a block-level id for CSS key
        styling but should not trigger reruns on tab change.
        """
        def __init__(
            self,
            *,
            default_tab_index: _builtins.int = ...,
            id: _builtins.str | None = ...,
        ) -> None: ...
        _HasFieldArgType: _TypeAlias = _typing.Literal["_id", b"_id", "id", b"id"]  # noqa: Y015
        def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
        _ClearFieldArgType: _TypeAlias = _typing.Literal["_id", b"_id", "default_tab_index", b"default_tab_index", "id", b"id"]  # noqa: Y015
        def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
        _WhichOneofReturnType__id: _TypeAlias = _typing.Literal["id"]  # noqa: Y015
        _WhichOneofArgType__id: _TypeAlias = _typing.Literal["_id", b"_id"]  # noqa: Y015
        def WhichOneof(self, oneof_group: _WhichOneofArgType__id) -> _WhichOneofReturnType__id | None: ...

    @_typing.final
    class Tab(_message.Message):
        DESCRIPTOR: _descriptor.Descriptor

        LABEL_FIELD_NUMBER: _builtins.int
        label: _builtins.str
        def __init__(
            self,
            *,
            label: _builtins.str = ...,
        ) -> None: ...
        _ClearFieldArgType: _TypeAlias = _typing.Literal["label", b"label"]  # noqa: Y015
        def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

    @_typing.final
    class Popover(_message.Message):
        DESCRIPTOR: _descriptor.Descriptor

        LABEL_FIELD_NUMBER: _builtins.int
        HELP_FIELD_NUMBER: _builtins.int
        DISABLED_FIELD_NUMBER: _builtins.int
        ICON_FIELD_NUMBER: _builtins.int
        TYPE_FIELD_NUMBER: _builtins.int
        OPEN_FIELD_NUMBER: _builtins.int
        ID_FIELD_NUMBER: _builtins.int
        label: _builtins.str
        help: _builtins.str
        disabled: _builtins.bool
        icon: _builtins.str
        type: _builtins.str
        open: _builtins.bool
        """Initial open state (used with on_change)."""
        id: _builtins.str
        """Widget ID for dynamic popovers. Only set when on_change="rerun",
        signaling the frontend to treat this as a stateful widget.
        When absent, the popover may still have a block-level id for CSS
        key styling but should not trigger reruns on toggle.
        """
        def __init__(
            self,
            *,
            label: _builtins.str = ...,
            help: _builtins.str = ...,
            disabled: _builtins.bool = ...,
            icon: _builtins.str = ...,
            type: _builtins.str = ...,
            open: _builtins.bool | None = ...,
            id: _builtins.str | None = ...,
        ) -> None: ...
        _HasFieldArgType: _TypeAlias = _typing.Literal["_id", b"_id", "_open", b"_open", "id", b"id", "open", b"open"]  # noqa: Y015
        def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
        _ClearFieldArgType: _TypeAlias = _typing.Literal["_id", b"_id", "_open", b"_open", "disabled", b"disabled", "help", b"help", "icon", b"icon", "id", b"id", "label", b"label", "open", b"open", "type", b"type"]  # noqa: Y015
        def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
        _WhichOneofReturnType__id: _TypeAlias = _typing.Literal["id"]  # noqa: Y015
        _WhichOneofArgType__id: _TypeAlias = _typing.Literal["_id", b"_id"]  # noqa: Y015
        _WhichOneofReturnType__open: _TypeAlias = _typing.Literal["open"]  # noqa: Y015
        _WhichOneofArgType__open: _TypeAlias = _typing.Literal["_open", b"_open"]  # noqa: Y015
        @_typing.overload
        def WhichOneof(self, oneof_group: _WhichOneofArgType__id) -> _WhichOneofReturnType__id | None: ...
        @_typing.overload
        def WhichOneof(self, oneof_group: _WhichOneofArgType__open) -> _WhichOneofReturnType__open | None: ...

    @_typing.final
    class ChatMessage(_message.Message):
        DESCRIPTOR: _descriptor.Descriptor

        class _AvatarType:
            ValueType = _typing.NewType("ValueType", _builtins.int)
            V: _TypeAlias = ValueType  # noqa: Y015

        class _AvatarTypeEnumTypeWrapper(_enum_type_wrapper._EnumTypeWrapper[Block.ChatMessage._AvatarType.ValueType], _builtins.type):
            DESCRIPTOR: _descriptor.EnumDescriptor
            IMAGE: Block.ChatMessage._AvatarType.ValueType  # 0
            EMOJI: Block.ChatMessage._AvatarType.ValueType  # 1
            ICON: Block.ChatMessage._AvatarType.ValueType  # 2

        class AvatarType(_AvatarType, metaclass=_AvatarTypeEnumTypeWrapper): ...
        IMAGE: Block.ChatMessage.AvatarType.ValueType  # 0
        EMOJI: Block.ChatMessage.AvatarType.ValueType  # 1
        ICON: Block.ChatMessage.AvatarType.ValueType  # 2

        NAME_FIELD_NUMBER: _builtins.int
        AVATAR_FIELD_NUMBER: _builtins.int
        AVATAR_TYPE_FIELD_NUMBER: _builtins.int
        name: _builtins.str
        avatar: _builtins.str
        avatar_type: Global___Block.ChatMessage.AvatarType.ValueType
        def __init__(
            self,
            *,
            name: _builtins.str = ...,
            avatar: _builtins.str = ...,
            avatar_type: Global___Block.ChatMessage.AvatarType.ValueType = ...,
        ) -> None: ...
        _ClearFieldArgType: _TypeAlias = _typing.Literal["avatar", b"avatar", "avatar_type", b"avatar_type", "name", b"name"]  # noqa: Y015
        def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

    VERTICAL_FIELD_NUMBER: _builtins.int
    HORIZONTAL_FIELD_NUMBER: _builtins.int
    COLUMN_FIELD_NUMBER: _builtins.int
    EXPANDABLE_FIELD_NUMBER: _builtins.int
    FORM_FIELD_NUMBER: _builtins.int
    TAB_CONTAINER_FIELD_NUMBER: _builtins.int
    TAB_FIELD_NUMBER: _builtins.int
    CHAT_MESSAGE_FIELD_NUMBER: _builtins.int
    POPOVER_FIELD_NUMBER: _builtins.int
    DIALOG_FIELD_NUMBER: _builtins.int
    FLEX_CONTAINER_FIELD_NUMBER: _builtins.int
    ALLOW_EMPTY_FIELD_NUMBER: _builtins.int
    ID_FIELD_NUMBER: _builtins.int
    HEIGHT_CONFIG_FIELD_NUMBER: _builtins.int
    WIDTH_CONFIG_FIELD_NUMBER: _builtins.int
    allow_empty: _builtins.bool
    id: _builtins.str
    @_builtins.property
    def vertical(self) -> Global___Block.Vertical: ...
    @_builtins.property
    def horizontal(self) -> Global___Block.Horizontal: ...
    @_builtins.property
    def column(self) -> Global___Block.Column: ...
    @_builtins.property
    def expandable(self) -> Global___Block.Expandable: ...
    @_builtins.property
    def form(self) -> Global___Block.Form: ...
    @_builtins.property
    def tab_container(self) -> Global___Block.TabContainer: ...
    @_builtins.property
    def tab(self) -> Global___Block.Tab: ...
    @_builtins.property
    def chat_message(self) -> Global___Block.ChatMessage: ...
    @_builtins.property
    def popover(self) -> Global___Block.Popover: ...
    @_builtins.property
    def dialog(self) -> Global___Block.Dialog: ...
    @_builtins.property
    def flex_container(self) -> Global___Block.FlexContainer: ...
    @_builtins.property
    def height_config(self) -> _HeightConfig_pb2.HeightConfig: ...
    @_builtins.property
    def width_config(self) -> _WidthConfig_pb2.WidthConfig: ...
    def __init__(
        self,
        *,
        vertical: Global___Block.Vertical | None = ...,
        horizontal: Global___Block.Horizontal | None = ...,
        column: Global___Block.Column | None = ...,
        expandable: Global___Block.Expandable | None = ...,
        form: Global___Block.Form | None = ...,
        tab_container: Global___Block.TabContainer | None = ...,
        tab: Global___Block.Tab | None = ...,
        chat_message: Global___Block.ChatMessage | None = ...,
        popover: Global___Block.Popover | None = ...,
        dialog: Global___Block.Dialog | None = ...,
        flex_container: Global___Block.FlexContainer | None = ...,
        allow_empty: _builtins.bool = ...,
        id: _builtins.str | None = ...,
        height_config: _HeightConfig_pb2.HeightConfig | None = ...,
        width_config: _WidthConfig_pb2.WidthConfig | None = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["_height_config", b"_height_config", "_id", b"_id", "_width_config", b"_width_config", "chat_message", b"chat_message", "column", b"column", "dialog", b"dialog", "expandable", b"expandable", "flex_container", b"flex_container", "form", b"form", "height_config", b"height_config", "horizontal", b"horizontal", "id", b"id", "popover", b"popover", "tab", b"tab", "tab_container", b"tab_container", "type", b"type", "vertical", b"vertical", "width_config", b"width_config"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["_height_config", b"_height_config", "_id", b"_id", "_width_config", b"_width_config", "allow_empty", b"allow_empty", "chat_message", b"chat_message", "column", b"column", "dialog", b"dialog", "expandable", b"expandable", "flex_container", b"flex_container", "form", b"form", "height_config", b"height_config", "horizontal", b"horizontal", "id", b"id", "popover", b"popover", "tab", b"tab", "tab_container", b"tab_container", "type", b"type", "vertical", b"vertical", "width_config", b"width_config"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType__height_config: _TypeAlias = _typing.Literal["height_config"]  # noqa: Y015
    _WhichOneofArgType__height_config: _TypeAlias = _typing.Literal["_height_config", b"_height_config"]  # noqa: Y015
    _WhichOneofReturnType__id: _TypeAlias = _typing.Literal["id"]  # noqa: Y015
    _WhichOneofArgType__id: _TypeAlias = _typing.Literal["_id", b"_id"]  # noqa: Y015
    _WhichOneofReturnType__width_config: _TypeAlias = _typing.Literal["width_config"]  # noqa: Y015
    _WhichOneofArgType__width_config: _TypeAlias = _typing.Literal["_width_config", b"_width_config"]  # noqa: Y015
    _WhichOneofReturnType_type: _TypeAlias = _typing.Literal["vertical", "horizontal", "column", "expandable", "form", "tab_container", "tab", "chat_message", "popover", "dialog", "flex_container"]  # noqa: Y015
    _WhichOneofArgType_type: _TypeAlias = _typing.Literal["type", b"type"]  # noqa: Y015
    @_typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__height_config) -> _WhichOneofReturnType__height_config | None: ...
    @_typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__id) -> _WhichOneofReturnType__id | None: ...
    @_typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__width_config) -> _WhichOneofReturnType__width_config | None: ...
    @_typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType_type) -> _WhichOneofReturnType_type | None: ...

Global___Block: _TypeAlias = Block  # noqa: Y015



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*!
Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from streamlit.proto import Alert_pb2 as _Alert_pb2
from streamlit.proto import AudioInput_pb2 as _AudioInput_pb2
from streamlit.proto import Audio_pb2 as _Audio_pb2
from streamlit.proto import Balloons_pb2 as _Balloons_pb2
from streamlit.proto import BidiComponent_pb2 as _BidiComponent_pb2
from streamlit.proto import ButtonGroup_pb2 as _ButtonGroup_pb2
from streamlit.proto import Button_pb2 as _Button_pb2
from streamlit.proto import CameraInput_pb2 as _CameraInput_pb2
from streamlit.proto import ChatInput_pb2 as _ChatInput_pb2
from streamlit.proto import Checkbox_pb2 as _Checkbox_pb2
from streamlit.proto import Code_pb2 as _Code_pb2
from streamlit.proto import ColorPicker_pb2 as _ColorPicker_pb2
from streamlit.proto import Components_pb2 as _Components_pb2
from streamlit.proto import Dataframe_pb2 as _Dataframe_pb2
from streamlit.proto import DateInput_pb2 as _DateInput_pb2
from streamlit.proto import DateTimeInput_pb2 as _DateTimeInput_pb2
from streamlit.proto import DeckGlJsonChart_pb2 as _DeckGlJsonChart_pb2
from streamlit.proto import DownloadButton_pb2 as _DownloadButton_pb2
from streamlit.proto import Empty_pb2 as _Empty_pb2
from streamlit.proto import Exception_pb2 as _Exception_pb2
from streamlit.proto import Favicon_pb2 as _Favicon_pb2
from streamlit.proto import Feedback_pb2 as _Feedback_pb2
from streamlit.proto import FileUploader_pb2 as _FileUploader_pb2
from streamlit.proto import GraphVizChart_pb2 as _GraphVizChart_pb2
from streamlit.proto import Heading_pb2 as _Heading_pb2
from streamlit.proto import HeightConfig_pb2 as _HeightConfig_pb2
from streamlit.proto import Help_pb2 as _Help_pb2
from streamlit.proto import Html_pb2 as _Html_pb2
from streamlit.proto import IFrame_pb2 as _IFrame_pb2
from streamlit.proto import Image_pb2 as _Image_pb2
from streamlit.proto import Json_pb2 as _Json_pb2
from streamlit.proto import LinkButton_pb2 as _LinkButton_pb2
from streamlit.proto import Markdown_pb2 as _Markdown_pb2
from streamlit.proto import MenuButton_pb2 as _MenuButton_pb2
from streamlit.proto import Metric_pb2 as _Metric_pb2
from streamlit.proto import MultiSelect_pb2 as _MultiSelect_pb2
from streamlit.proto import NumberInput_pb2 as _NumberInput_pb2
from streamlit.proto import PageLink_pb2 as _PageLink_pb2
from streamlit.proto import PlotlyChart_pb2 as _PlotlyChart_pb2
from streamlit.proto import Progress_pb2 as _Progress_pb2
from streamlit.proto import Radio_pb2 as _Radio_pb2
from streamlit.proto import Selectbox_pb2 as _Selectbox_pb2
from streamlit.proto import Skeleton_pb2 as _Skeleton_pb2
from streamlit.proto import Slider_pb2 as _Slider_pb2
from streamlit.proto import Snow_pb2 as _Snow_pb2
from streamlit.proto import Space_pb2 as _Space_pb2
from streamlit.proto import Spinner_pb2 as _Spinner_pb2
from streamlit.proto import Table_pb2 as _Table_pb2
from streamlit.proto import TextAlignmentConfig_pb2 as _TextAlignmentConfig_pb2
from streamlit.proto import TextArea_pb2 as _TextArea_pb2
from streamlit.proto import TextInput_pb2 as _TextInput_pb2
from streamlit.proto import Text_pb2 as _Text_pb2
from streamlit.proto import TimeInput_pb2 as _TimeInput_pb2
from streamlit.proto import Toast_pb2 as _Toast_pb2
from streamlit.proto import VegaLiteChart_pb2 as _VegaLiteChart_pb2
from streamlit.proto import Video_pb2 as _Video_pb2
from streamlit.proto import WidthConfig_pb2 as _WidthConfig_pb2
import builtins as _builtins
import sys
import typing as _typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: _descriptor.FileDescriptor

@_typing.final
class Element(_message.Message):
    """An element which can be displayed on the screen."""

    DESCRIPTOR: _descriptor.Descriptor

    HEIGHT_CONFIG_FIELD_NUMBER: _builtins.int
    WIDTH_CONFIG_FIELD_NUMBER: _builtins.int
    TEXT_ALIGNMENT_CONFIG_FIELD_NUMBER: _builtins.int
    ALERT_FIELD_NUMBER: _builtins.int
    DATAFRAME_FIELD_NUMBER: _builtins.int
    TABLE_FIELD_NUMBER: _builtins.int
    VEGA_LITE_CHART_FIELD_NUMBER: _builtins.int
    AUDIO_FIELD_NUMBER: _builtins.int
    AUDIO_INPUT_FIELD_NUMBER: _builtins.int
    BALLOONS_FIELD_NUMBER: _builtins.int
    BIDI_COMPONENT_FIELD_NUMBER: _builtins.int
    BUTTON_FIELD_NUMBER: _builtins.int
    BUTTON_GROUP_FIELD_NUMBER: _builtins.int
    DOWNLOAD_BUTTON_FIELD_NUMBER: _builtins.int
    CAMERA_INPUT_FIELD_NUMBER: _builtins.int
    CHAT_INPUT_FIELD_NUMBER: _builtins.int
    CHECKBOX_FIELD_NUMBER: _builtins.int
    COLOR_PICKER_FIELD_NUMBER: _builtins.int
    COMPONENT_INSTANCE_FIELD_NUMBER: _builtins.int
    DATE_INPUT_FIELD_NUMBER: _builtins.int
    DECK_GL_JSON_CHART_FIELD_NUMBER: _builtins.int
    HELP_INFO_FIELD_NUMBER: _builtins.int
    EMPTY_FIELD_NUMBER: _builtins.int
    EXCEPTION_FIELD_NUMBER: _builtins.int
    FAVICON_FIELD_NUMBER: _builtins.int
    FEEDBACK_FIELD_NUMBER: _builtins.int
    FILE_UPLOADER_FIELD_NUMBER: _builtins.int
    GRAPHVIZ_CHART_FIELD_NUMBER: _builtins.int
    HTML_FIELD_NUMBER: _builtins.int
    IFRAME_FIELD_NUMBER: _builtins.int
    IMGS_FIELD_NUMBER: _builtins.int
    JSON_FIELD_NUMBER: _builtins.int
    LINK_BUTTON_FIELD_NUMBER: _builtins.int
    MARKDOWN_FIELD_NUMBER: _builtins.int
    METRIC_FIELD_NUMBER: _builtins.int
    MULTISELECT_FIELD_NUMBER: _builtins.int
    NUMBER_INPUT_FIELD_NUMBER: _builtins.int
    PAGE_LINK_FIELD_NUMBER: _builtins.int
    PLOTLY_CHART_FIELD_NUMBER: _builtins.int
    PROGRESS_FIELD_NUMBER: _builtins.int
    RADIO_FIELD_NUMBER: _builtins.int
    SELECTBOX_FIELD_NUMBER: _builtins.int
    SKELETON_FIELD_NUMBER: _builtins.int
    SLIDER_FIELD_NUMBER: _builtins.int
    SNOW_FIELD_NUMBER: _builtins.int
    SPACE_FIELD_NUMBER: _builtins.int
    SPINNER_FIELD_NUMBER: _builtins.int
    TEXT_FIELD_NUMBER: _builtins.int
    TEXT_AREA_FIELD_NUMBER: _builtins.int
    TEXT_INPUT_FIELD_NUMBER: _builtins.int
    TIME_INPUT_FIELD_NUMBER: _builtins.int
    DATE_TIME_INPUT_FIELD_NUMBER: _builtins.int
    TOAST_FIELD_NUMBER: _builtins.int
    VIDEO_FIELD_NUMBER: _builtins.int
    HEADING_FIELD_NUMBER: _builtins.int
    CODE_FIELD_NUMBER: _builtins.int
    MENU_BUTTON_FIELD_NUMBER: _builtins.int
    @_builtins.property
    def height_config(self) -> _HeightConfig_pb2.HeightConfig:
        """Layout configuration for elements"""

    @_builtins.property
    def width_config(self) -> _WidthConfig_pb2.WidthConfig: ...
    @_builtins.property
    def text_alignment_config(self) -> _TextAlignmentConfig_pb2.TextAlignmentConfig: ...
    @_builtins.property
    def alert(self) -> _Alert_pb2.Alert: ...
    @_builtins.property
    def dataframe(self) -> _Dataframe_pb2.Dataframe: ...
    @_builtins.property
    def table(self) -> _Table_pb2.Table: ...
    @_builtins.property
    def vega_lite_chart(self) -> _VegaLiteChart_pb2.VegaLiteChart: ...
    @_builtins.property
    def audio(self) -> _Audio_pb2.Audio: ...
    @_builtins.property
    def audio_input(self) -> _AudioInput_pb2.AudioInput: ...
    @_builtins.property
    def balloons(self) -> _Balloons_pb2.Balloons: ...
    @_builtins.property
    def bidi_component(self) -> _BidiComponent_pb2.BidiComponent: ...
    @_builtins.property
    def button(self) -> _Button_pb2.Button: ...
    @_builtins.property
    def button_group(self) -> _ButtonGroup_pb2.ButtonGroup: ...
    @_builtins.property
    def download_button(self) -> _DownloadButton_pb2.DownloadButton: ...
    @_builtins.property
    def camera_input(self) -> _CameraInput_pb2.CameraInput: ...
    @_builtins.property
    def chat_input(self) -> _ChatInput_pb2.ChatInput: ...
    @_builtins.property
    def checkbox(self) -> _Checkbox_pb2.Checkbox: ...
    @_builtins.property
    def color_picker(self) -> _ColorPicker_pb2.ColorPicker: ...
    @_builtins.property
    def component_instance(self) -> _Components_pb2.ComponentInstance: ...
    @_builtins.property
    def date_input(self) -> _DateInput_pb2.DateInput: ...
    @_builtins.property
    def deck_gl_json_chart(self) -> _DeckGlJsonChart_pb2.DeckGlJsonChart: ...
    @_builtins.property
    def help_info(self) -> _Help_pb2.Help: ...
    @_builtins.property
    def empty(self) -> _Empty_pb2.Empty: ...
    @_builtins.property
    def exception(self) -> _Exception_pb2.Exception: ...
    @_builtins.property
    def favicon(self) -> _Favicon_pb2.Favicon: ...
    @_builtins.property
    def feedback(self) -> _Feedback_pb2.Feedback: ...
    @_builtins.property
    def file_uploader(self) -> _FileUploader_pb2.FileUploader: ...
    @_builtins.property
    def graphviz_chart(self) -> _GraphVizChart_pb2.GraphVizChart: ...
    @_builtins.property
    def html(self) -> _Html_pb2.Html: ...
    @_builtins.property
    def iframe(self) -> _IFrame_pb2.IFrame: ...
    @_builtins.property
    def imgs(self) -> _Image_pb2.ImageList: ...
    @_builtins.property
    def json(self) -> _Json_pb2.Json: ...
    @_builtins.property
    def link_button(self) -> _LinkButton_pb2.LinkButton: ...
    @_builtins.property
    def markdown(self) -> _Markdown_pb2.Markdown: ...
    @_builtins.property
    def metric(self) -> _Metric_pb2.Metric: ...
    @_builtins.property
    def multiselect(self) -> _MultiSelect_pb2.MultiSelect: ...
    @_builtins.property
    def number_input(self) -> _NumberInput_pb2.NumberInput: ...
    @_builtins.property
    def page_link(self) -> _PageLink_pb2.PageLink: ...
    @_builtins.property
    def plotly_chart(self) -> _PlotlyChart_pb2.PlotlyChart: ...
    @_builtins.property
    def progress(self) -> _Progress_pb2.Progress: ...
    @_builtins.property
    def radio(self) -> _Radio_pb2.Radio: ...
    @_builtins.property
    def selectbox(self) -> _Selectbox_pb2.Selectbox: ...
    @_builtins.property
    def skeleton(self) -> _Skeleton_pb2.Skeleton: ...
    @_builtins.property
    def slider(self) -> _Slider_pb2.Slider: ...
    @_builtins.property
    def snow(self) -> _Snow_pb2.Snow: ...
    @_builtins.property
    def space(self) -> _Space_pb2.Space: ...
    @_builtins.property
    def spinner(self) -> _Spinner_pb2.Spinner: ...
    @_builtins.property
    def text(self) -> _Text_pb2.Text: ...
    @_builtins.property
    def text_area(self) -> _TextArea_pb2.TextArea: ...
    @_builtins.property
    def text_input(self) -> _TextInput_pb2.TextInput: ...
    @_builtins.property
    def time_input(self) -> _TimeInput_pb2.TimeInput: ...
    @_builtins.property
    def date_time_input(self) -> _DateTimeInput_pb2.DateTimeInput: ...
    @_builtins.property
    def toast(self) -> _Toast_pb2.Toast: ...
    @_builtins.property
    def video(self) -> _Video_pb2.Video: ...
    @_builtins.property
    def heading(self) -> _Heading_pb2.Heading: ...
    @_builtins.property
    def code(self) -> _Code_pb2.Code: ...
    @_builtins.property
    def menu_button(self) -> _MenuButton_pb2.MenuButton:
        """Next ID: 65"""

    def __init__(
        self,
        *,
        height_config: _HeightConfig_pb2.HeightConfig | None = ...,
        width_config: _WidthConfig_pb2.WidthConfig | None = ...,
        text_alignment_config: _TextAlignmentConfig_pb2.TextAlignmentConfig | None = ...,
        alert: _Alert_pb2.Alert | None = ...,
        dataframe: _Dataframe_pb2.Dataframe | None = ...,
        table: _Table_pb2.Table | None = ...,
        vega_lite_chart: _VegaLiteChart_pb2.VegaLiteChart | None = ...,
        audio: _Audio_pb2.Audio | None = ...,
        audio_input: _AudioInput_pb2.AudioInput | None = ...,
        balloons: _Balloons_pb2.Balloons | None = ...,
        bidi_component: _BidiComponent_pb2.BidiComponent | None = ...,
        button: _Button_pb2.Button | None = ...,
        button_group: _ButtonGroup_pb2.ButtonGroup | None = ...,
        download_button: _DownloadButton_pb2.DownloadButton | None = ...,
        camera_input: _CameraInput_pb2.CameraInput | None = ...,
        chat_input: _ChatInput_pb2.ChatInput | None = ...,
        checkbox: _Checkbox_pb2.Checkbox | None = ...,
        color_picker: _ColorPicker_pb2.ColorPicker | None = ...,
        component_instance: _Components_pb2.ComponentInstance | None = ...,
        date_input: _DateInput_pb2.DateInput | None = ...,
        deck_gl_json_chart: _DeckGlJsonChart_pb2.DeckGlJsonChart | None = ...,
        help_info: _Help_pb2.Help | None = ...,
        empty: _Empty_pb2.Empty | None = ...,
        exception: _Exception_pb2.Exception | None = ...,
        favicon: _Favicon_pb2.Favicon | None = ...,
        feedback: _Feedback_pb2.Feedback | None = ...,
        file_uploader: _FileUploader_pb2.FileUploader | None = ...,
        graphviz_chart: _GraphVizChart_pb2.GraphVizChart | None = ...,
        html: _Html_pb2.Html | None = ...,
        iframe: _IFrame_pb2.IFrame | None = ...,
        imgs: _Image_pb2.ImageList | None = ...,
        json: _Json_pb2.Json | None = ...,
        link_button: _LinkButton_pb2.LinkButton | None = ...,
        markdown: _Markdown_pb2.Markdown | None = ...,
        metric: _Metric_pb2.Metric | None = ...,
        multiselect: _MultiSelect_pb2.MultiSelect | None = ...,
        number_input: _NumberInput_pb2.NumberInput | None = ...,
        page_link: _PageLink_pb2.PageLink | None = ...,
        plotly_chart: _PlotlyChart_pb2.PlotlyChart | None = ...,
        progress: _Progress_pb2.Progress | None = ...,
        radio: _Radio_pb2.Radio | None = ...,
        selectbox: _Selectbox_pb2.Selectbox | None = ...,
        skeleton: _Skeleton_pb2.Skeleton | None = ...,
        slider: _Slider_pb2.Slider | None = ...,
        snow: _Snow_pb2.Snow | None = ...,
        space: _Space_pb2.Space | None = ...,
        spinner: _Spinner_pb2.Spinner | None = ...,
        text: _Text_pb2.Text | None = ...,
        text_area: _TextArea_pb2.TextArea | None = ...,
        text_input: _TextInput_pb2.TextInput | None = ...,
        time_input: _TimeInput_pb2.TimeInput | None = ...,
        date_time_input: _DateTimeInput_pb2.DateTimeInput | None = ...,
        toast: _Toast_pb2.Toast | None = ...,
        video: _Video_pb2.Video | None = ...,
        heading: _Heading_pb2.Heading | None = ...,
        code: _Code_pb2.Code | None = ...,
        menu_button: _MenuButton_pb2.MenuButton | None = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["_height_config", b"_height_config", "_text_alignment_config", b"_text_alignment_config", "_width_config", b"_width_config", "alert", b"alert", "audio", b"audio", "audio_input", b"audio_input", "balloons", b"balloons", "bidi_component", b"bidi_component", "button", b"button", "button_group", b"button_group", "camera_input", b"camera_input", "chat_input", b"chat_input", "checkbox", b"checkbox", "code", b"code", "color_picker", b"color_picker", "component_instance", b"component_instance", "dataframe", b"dataframe", "date_input", b"date_input", "date_time_input", b"date_time_input", "deck_gl_json_chart", b"deck_gl_json_chart", "download_button", b"download_button", "empty", b"empty", "exception", b"exception", "favicon", b"favicon", "feedback", b"feedback", "file_uploader", b"file_uploader", "graphviz_chart", b"graphviz_chart", "heading", b"heading", "height_config", b"height_config", "help_info", b"help_info", "html", b"html", "iframe", b"iframe", "imgs", b"imgs", "json", b"json", "link_button", b"link_button", "markdown", b"markdown", "menu_button", b"menu_button", "metric", b"metric", "multiselect", b"multiselect", "number_input", b"number_input", "page_link", b"page_link", "plotly_chart", b"plotly_chart", "progress", b"progress", "radio", b"radio", "selectbox", b"selectbox", "skeleton", b"skeleton", "slider", b"slider", "snow", b"snow", "space", b"space", "spinner", b"spinner", "table", b"table", "text", b"text", "text_alignment_config", b"text_alignment_config", "text_area", b"text_area", "text_input", b"text_input", "time_input", b"time_input", "toast", b"toast", "type", b"type", "vega_lite_chart", b"vega_lite_chart", "video", b"video", "width_config", b"width_config"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["_height_config", b"_height_config", "_text_alignment_config", b"_text_alignment_config", "_width_config", b"_width_config", "alert", b"alert", "audio", b"audio", "audio_input", b"audio_input", "balloons", b"balloons", "bidi_component", b"bidi_component", "button", b"button", "button_group", b"button_group", "camera_input", b"camera_input", "chat_input", b"chat_input", "checkbox", b"checkbox", "code", b"code", "color_picker", b"color_picker", "component_instance", b"component_instance", "dataframe", b"dataframe", "date_input", b"date_input", "date_time_input", b"date_time_input", "deck_gl_json_chart", b"deck_gl_json_chart", "download_button", b"download_button", "empty", b"empty", "exception", b"exception", "favicon", b"favicon", "feedback", b"feedback", "file_uploader", b"file_uploader", "graphviz_chart", b"graphviz_chart", "heading", b"heading", "height_config", b"height_config", "help_info", b"help_info", "html", b"html", "iframe", b"iframe", "imgs", b"imgs", "json", b"json", "link_button", b"link_button", "markdown", b"markdown", "menu_button", b"menu_button", "metric", b"metric", "multiselect", b"multiselect", "number_input", b"number_input", "page_link", b"page_link", "plotly_chart", b"plotly_chart", "progress", b"progress", "radio", b"radio", "selectbox", b"selectbox", "skeleton", b"skeleton", "slider", b"slider", "snow", b"snow", "space", b"space", "spinner", b"spinner", "table", b"table", "text", b"text", "text_alignment_config", b"text_alignment_config", "text_area", b"text_area", "text_input", b"text_input", "time_input", b"time_input", "toast", b"toast", "type", b"type", "vega_lite_chart", b"vega_lite_chart", "video", b"video", "width_config", b"width_config"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType__height_config: _TypeAlias = _typing.Literal["height_config"]  # noqa: Y015
    _WhichOneofArgType__height_config: _TypeAlias = _typing.Literal["_height_config", b"_height_config"]  # noqa: Y015
    _WhichOneofReturnType__text_alignment_config: _TypeAlias = _typing.Literal["text_alignment_config"]  # noqa: Y015
    _WhichOneofArgType__text_alignment_config: _TypeAlias = _typing.Literal["_text_alignment_config", b"_text_alignment_config"]  # noqa: Y015
    _WhichOneofReturnType__width_config: _TypeAlias = _typing.Literal["width_config"]  # noqa: Y015
    _WhichOneofArgType__width_config: _TypeAlias = _typing.Literal["_width_config", b"_width_config"]  # noqa: Y015
    _WhichOneofReturnType_type: _TypeAlias = _typing.Literal["alert", "dataframe", "table", "vega_lite_chart", "audio", "audio_input", "balloons", "bidi_component", "button", "button_group", "download_button", "camera_input", "chat_input", "checkbox", "color_picker", "component_instance", "date_input", "deck_gl_json_chart", "help_info", "empty", "exception", "favicon", "feedback", "file_uploader", "graphviz_chart", "html", "iframe", "imgs", "json", "link_button", "markdown", "metric", "multiselect", "number_input", "page_link", "plotly_chart", "progress", "radio", "selectbox", "skeleton", "slider", "snow", "space", "spinner", "text", "text_area", "text_input", "time_input", "date_time_input", "toast", "video", "heading", "code", "menu_button"]  # noqa: Y015
    _WhichOneofArgType_type: _TypeAlias = _typing.Literal["type", b"type"]  # noqa: Y015
    @_typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__height_config) -> _WhichOneofReturnType__height_config | None: ...
    @_typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__text_alignment_config) -> _WhichOneofReturnType__text_alignment_config | None: ...
    @_typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__width_config) -> _WhichOneofReturnType__width_config | None: ...
    @_typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType_type) -> _WhichOneofReturnType_type | None: ...

Global___Element: _TypeAlias = Element  # noqa: Y015



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*!
Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from collections import abc as _abc
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf.internal import containers as _containers
import builtins as _builtins
import sys
import typing as _typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: _descriptor.FileDescriptor

@_typing.final
class StringArray(_message.Message):
    """Message types that are common to multiple protobufs."""

    DESCRIPTOR: _descriptor.Descriptor

    DATA_FIELD_NUMBER: _builtins.int
    @_builtins.property
    def data(self) -> _containers.RepeatedScalarFieldContainer[_builtins.str]: ...
    def __init__(
        self,
        *,
        data: _abc.Iterable[_builtins.str] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["data", b"data"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___StringArray: _TypeAlias = StringArray  # noqa: Y015

@_typing.final
class DoubleArray(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    DATA_FIELD_NUMBER: _builtins.int
    @_builtins.property
    def data(self) -> _containers.RepeatedScalarFieldContainer[_builtins.float]: ...
    def __init__(
        self,
        *,
        data: _abc.Iterable[_builtins.float] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["data", b"data"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___DoubleArray: _TypeAlias = DoubleArray  # noqa: Y015

@_typing.final
class Int32Array(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    DATA_FIELD_NUMBER: _builtins.int
    @_builtins.property
    def data(self) -> _containers.RepeatedScalarFieldContainer[_builtins.int]: ...
    def __init__(
        self,
        *,
        data: _abc.Iterable[_builtins.int] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["data", b"data"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___Int32Array: _TypeAlias = Int32Array  # noqa: Y015

@_typing.final
class Int64Array(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    DATA_FIELD_NUMBER: _builtins.int
    @_builtins.property
    def data(self) -> _containers.RepeatedScalarFieldContainer[_builtins.int]: ...
    def __init__(
        self,
        *,
        data: _abc.Iterable[_builtins.int] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["data", b"data"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___Int64Array: _TypeAlias = Int64Array  # noqa: Y015

@_typing.final
class SInt64Array(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    DATA_FIELD_NUMBER: _builtins.int
    @_builtins.property
    def data(self) -> _containers.RepeatedScalarFieldContainer[_builtins.int]: ...
    def __init__(
        self,
        *,
        data: _abc.Iterable[_builtins.int] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["data", b"data"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___SInt64Array: _TypeAlias = SInt64Array  # noqa: Y015

@_typing.final
class UInt32Array(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    DATA_FIELD_NUMBER: _builtins.int
    @_builtins.property
    def data(self) -> _containers.RepeatedScalarFieldContainer[_builtins.int]: ...
    def __init__(
        self,
        *,
        data: _abc.Iterable[_builtins.int] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["data", b"data"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___UInt32Array: _TypeAlias = UInt32Array  # noqa: Y015

@_typing.final
class StringTriggerValue(_message.Message):
    """A trigger value that carries a single string payload.
    Used for trigger-based widgets that need to return a string value
    (e.g., menu button selections). The value auto-resets after each script run.
    """

    DESCRIPTOR: _descriptor.Descriptor

    DATA_FIELD_NUMBER: _builtins.int
    data: _builtins.str
    def __init__(
        self,
        *,
        data: _builtins.str | None = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["_data", b"_data", "data", b"data"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["_data", b"_data", "data", b"data"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType__data: _TypeAlias = _typing.Literal["data"]  # noqa: Y015
    _WhichOneofArgType__data: _TypeAlias = _typing.Literal["_data", b"_data"]  # noqa: Y015
    def WhichOneof(self, oneof_group: _WhichOneofArgType__data) -> _WhichOneofReturnType__data | None: ...

Global___StringTriggerValue: _TypeAlias = StringTriggerValue  # noqa: Y015

@_typing.final
class FileURLsRequest(_message.Message):
    """NOTE: The FileURLsRequest, FileURLs, and FileURLsResponse message types
    must remain stable as some external services rely on them to support
    `st.file_uploader`. These types aren't completely set in stone, but changing
    them requires a good amount of effort so should be avoided if possible.
    """

    DESCRIPTOR: _descriptor.Descriptor

    REQUEST_ID_FIELD_NUMBER: _builtins.int
    FILE_NAMES_FIELD_NUMBER: _builtins.int
    SESSION_ID_FIELD_NUMBER: _builtins.int
    request_id: _builtins.str
    session_id: _builtins.str
    @_builtins.property
    def file_names(self) -> _containers.RepeatedScalarFieldContainer[_builtins.str]: ...
    def __init__(
        self,
        *,
        request_id: _builtins.str = ...,
        file_names: _abc.Iterable[_builtins.str] | None = ...,
        session_id: _builtins.str = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["file_names", b"file_names", "request_id", b"request_id", "session_id", b"session_id"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___FileURLsRequest: _TypeAlias = FileURLsRequest  # noqa: Y015

@_typing.final
class FileURLs(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    FILE_ID_FIELD_NUMBER: _builtins.int
    UPLOAD_URL_FIELD_NUMBER: _builtins.int
    DELETE_URL_FIELD_NUMBER: _builtins.int
    file_id: _builtins.str
    upload_url: _builtins.str
    delete_url: _builtins.str
    def __init__(
        self,
        *,
        file_id: _builtins.str = ...,
        upload_url: _builtins.str = ...,
        delete_url: _builtins.str = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["delete_url", b"delete_url", "file_id", b"file_id", "upload_url", b"upload_url"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___FileURLs: _TypeAlias = FileURLs  # noqa: Y015

@_typing.final
class FileURLsResponse(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    RESPONSE_ID_FIELD_NUMBER: _builtins.int
    FILE_URLS_FIELD_NUMBER: _builtins.int
    ERROR_MSG_FIELD_NUMBER: _builtins.int
    response_id: _builtins.str
    error_msg: _builtins.str
    @_builtins.property
    def file_urls(self) -> _containers.RepeatedCompositeFieldContainer[Global___FileURLs]: ...
    def __init__(
        self,
        *,
        response_id: _builtins.str = ...,
        file_urls: _abc.Iterable[Global___FileURLs] | None = ...,
        error_msg: _builtins.str = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["error_msg", b"error_msg", "file_urls", b"file_urls", "response_id", b"response_id"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___FileURLsResponse: _TypeAlias = FileURLsResponse  # noqa: Y015

@_typing.final
class UploadedFileInfo(_message.Message):
    """Information on a file uploaded via the file_uploader widget."""

    DESCRIPTOR: _descriptor.Descriptor

    NAME_FIELD_NUMBER: _builtins.int
    SIZE_FIELD_NUMBER: _builtins.int
    FILE_ID_FIELD_NUMBER: _builtins.int
    FILE_URLS_FIELD_NUMBER: _builtins.int
    name: _builtins.str
    size: _builtins.int
    """The size of this file in bytes."""
    file_id: _builtins.str
    """ID that can be used to retrieve a file."""
    @_builtins.property
    def file_urls(self) -> Global___FileURLs:
        """Metadata containing information about file_urls."""

    def __init__(
        self,
        *,
        name: _builtins.str = ...,
        size: _builtins.int = ...,
        file_id: _builtins.str = ...,
        file_urls: Global___FileURLs | None = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["file_urls", b"file_urls"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["file_id", b"file_id", "file_urls", b"file_urls", "name", b"name", "size", b"size"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___UploadedFileInfo: _TypeAlias = UploadedFileInfo  # noqa: Y015

@_typing.final
class FileUploaderState(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    UPLOADED_FILE_INFO_FIELD_NUMBER: _builtins.int
    @_builtins.property
    def uploaded_file_info(self) -> _containers.RepeatedCompositeFieldContainer[Global___UploadedFileInfo]: ...
    def __init__(
        self,
        *,
        uploaded_file_info: _abc.Iterable[Global___UploadedFileInfo] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["uploaded_file_info", b"uploaded_file_info"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___FileUploaderState: _TypeAlias = FileUploaderState  # noqa: Y015

@_typing.final
class ChatInputValue(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    DATA_FIELD_NUMBER: _builtins.int
    FILE_UPLOADER_STATE_FIELD_NUMBER: _builtins.int
    AUDIO_FILE_INFO_FIELD_NUMBER: _builtins.int
    data: _builtins.str
    @_builtins.property
    def file_uploader_state(self) -> Global___FileUploaderState: ...
    @_builtins.property
    def audio_file_info(self) -> Global___UploadedFileInfo: ...
    def __init__(
        self,
        *,
        data: _builtins.str | None = ...,
        file_uploader_state: Global___FileUploaderState | None = ...,
        audio_file_info: Global___UploadedFileInfo | None = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["_audio_file_info", b"_audio_file_info", "_data", b"_data", "_file_uploader_state", b"_file_uploader_state", "audio_file_info", b"audio_file_info", "data", b"data", "file_uploader_state", b"file_uploader_state"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["_audio_file_info", b"_audio_file_info", "_data", b"_data", "_file_uploader_state", b"_file_uploader_state", "audio_file_info", b"audio_file_info", "data", b"data", "file_uploader_state", b"file_uploader_state"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType__audio_file_info: _TypeAlias = _typing.Literal["audio_file_info"]  # noqa: Y015
    _WhichOneofArgType__audio_file_info: _TypeAlias = _typing.Literal["_audio_file_info", b"_audio_file_info"]  # noqa: Y015
    _WhichOneofReturnType__data: _TypeAlias = _typing.Literal["data"]  # noqa: Y015
    _WhichOneofArgType__data: _TypeAlias = _typing.Literal["_data", b"_data"]  # noqa: Y015
    _WhichOneofReturnType__file_uploader_state: _TypeAlias = _typing.Literal["file_uploader_state"]  # noqa: Y015
    _WhichOneofArgType__file_uploader_state: _TypeAlias = _typing.Literal["_file_uploader_state", b"_file_uploader_state"]  # noqa: Y015
    @_typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__audio_file_info) -> _WhichOneofReturnType__audio_file_info | None: ...
    @_typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__data) -> _WhichOneofReturnType__data | None: ...
    @_typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__file_uploader_state) -> _WhichOneofReturnType__file_uploader_state | None: ...

Global___ChatInputValue: _TypeAlias = ChatInputValue  # noqa: Y015



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*!
Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from collections import abc as _abc
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from streamlit.proto import WidthConfig_pb2 as _WidthConfig_pb2
import builtins as _builtins
import sys
import typing as _typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: _descriptor.FileDescriptor

@_typing.final
class SubtitleTrack(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    LABEL_FIELD_NUMBER: _builtins.int
    URL_FIELD_NUMBER: _builtins.int
    label: _builtins.str
    """Label for the subtitle e.g. "English" or "Spanish"."""
    url: _builtins.str
    def __init__(
        self,
        *,
        label: _builtins.str = ...,
        url: _builtins.str = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["label", b"label", "url", b"url"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___SubtitleTrack: _TypeAlias = SubtitleTrack  # noqa: Y015

@_typing.final
class Video(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    class _Type:
        ValueType = _typing.NewType("ValueType", _builtins.int)
        V: _TypeAlias = ValueType  # noqa: Y015

    class _TypeEnumTypeWrapper(_enum_type_wrapper._EnumTypeWrapper[Video._Type.ValueType], _builtins.type):
        DESCRIPTOR: _descriptor.EnumDescriptor
        UNUSED: Video._Type.ValueType  # 0
        """This should always exist."""
        NATIVE: Video._Type.ValueType  # 1
        YOUTUBE_IFRAME: Video._Type.ValueType  # 2

    class Type(_Type, metaclass=_TypeEnumTypeWrapper): ...
    UNUSED: Video.Type.ValueType  # 0
    """This should always exist."""
    NATIVE: Video.Type.ValueType  # 1
    YOUTUBE_IFRAME: Video.Type.ValueType  # 2

    URL_FIELD_NUMBER: _builtins.int
    START_TIME_FIELD_NUMBER: _builtins.int
    TYPE_FIELD_NUMBER: _builtins.int
    SUBTITLES_FIELD_NUMBER: _builtins.int
    END_TIME_FIELD_NUMBER: _builtins.int
    LOOP_FIELD_NUMBER: _builtins.int
    AUTOPLAY_FIELD_NUMBER: _builtins.int
    MUTED_FIELD_NUMBER: _builtins.int
    ID_FIELD_NUMBER: _builtins.int
    WIDTH_CONFIG_FIELD_NUMBER: _builtins.int
    url: _builtins.str
    """A url pointing to a video file"""
    start_time: _builtins.int
    """The currentTime attribute of the HTML <video> tag's <source> subtag."""
    type: Global___Video.Type.ValueType
    """Type affects how browser wraps the video in tags: plain HTML5, YouTube..."""
    end_time: _builtins.int
    """The time at which the video should stop playing. If not specified, plays to the end."""
    loop: _builtins.bool
    """Indicates whether the video should start over from the beginning once it ends."""
    autoplay: _builtins.bool
    muted: _builtins.bool
    id: _builtins.str
    @_builtins.property
    def subtitles(self) -> _containers.RepeatedCompositeFieldContainer[Global___SubtitleTrack]:
        """Repeated field for subtitle tracks"""

    @_builtins.property
    def width_config(self) -> _WidthConfig_pb2.WidthConfig: ...
    def __init__(
        self,
        *,
        url: _builtins.str = ...,
        start_time: _builtins.int = ...,
        type: Global___Video.Type.ValueType = ...,
        subtitles: _abc.Iterable[Global___SubtitleTrack] | None = ...,
        end_time: _builtins.int = ...,
        loop: _builtins.bool = ...,
        autoplay: _builtins.bool = ...,
        muted: _builtins.bool = ...,
        id: _builtins.str = ...,
        width_config: _WidthConfig_pb2.WidthConfig | None = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["_width_config", b"_width_config", "width_config", b"width_config"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["_width_config", b"_width_config", "autoplay", b"autoplay", "end_time", b"end_time", "id", b"id", "loop", b"loop", "muted", b"muted", "start_time", b"start_time", "subtitles", b"subtitles", "type", b"type", "url", b"url", "width_config", b"width_config"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType__width_config: _TypeAlias = _typing.Literal["width_config"]  # noqa: Y015
    _WhichOneofArgType__width_config: _TypeAlias = _typing.Literal["_width_config", b"_width_config"]  # noqa: Y015
    def WhichOneof(self, oneof_group: _WhichOneofArgType__width_config) -> _WhichOneofReturnType__width_config | None: ...

Global___Video: _TypeAlias = Video  # noqa: Y015



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*!
Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from collections import abc as _abc
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from streamlit.proto import LabelVisibility_pb2 as _LabelVisibility_pb2
import builtins as _builtins
import sys
import typing as _typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: _descriptor.FileDescriptor

@_typing.final
class Metric(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    class _MetricDirection:
        ValueType = _typing.NewType("ValueType", _builtins.int)
        V: _TypeAlias = ValueType  # noqa: Y015

    class _MetricDirectionEnumTypeWrapper(_enum_type_wrapper._EnumTypeWrapper[Metric._MetricDirection.ValueType], _builtins.type):
        DESCRIPTOR: _descriptor.EnumDescriptor
        DOWN: Metric._MetricDirection.ValueType  # 0
        UP: Metric._MetricDirection.ValueType  # 1
        NONE: Metric._MetricDirection.ValueType  # 2

    class MetricDirection(_MetricDirection, metaclass=_MetricDirectionEnumTypeWrapper): ...
    DOWN: Metric.MetricDirection.ValueType  # 0
    UP: Metric.MetricDirection.ValueType  # 1
    NONE: Metric.MetricDirection.ValueType  # 2

    class _MetricColor:
        ValueType = _typing.NewType("ValueType", _builtins.int)
        V: _TypeAlias = ValueType  # noqa: Y015

    class _MetricColorEnumTypeWrapper(_enum_type_wrapper._EnumTypeWrapper[Metric._MetricColor.ValueType], _builtins.type):
        DESCRIPTOR: _descriptor.EnumDescriptor
        RED: Metric._MetricColor.ValueType  # 0
        GREEN: Metric._MetricColor.ValueType  # 1
        GRAY: Metric._MetricColor.ValueType  # 2
        ORANGE: Metric._MetricColor.ValueType  # 3
        YELLOW: Metric._MetricColor.ValueType  # 4
        BLUE: Metric._MetricColor.ValueType  # 5
        VIOLET: Metric._MetricColor.ValueType  # 6
        PRIMARY: Metric._MetricColor.ValueType  # 7

    class MetricColor(_MetricColor, metaclass=_MetricColorEnumTypeWrapper): ...
    RED: Metric.MetricColor.ValueType  # 0
    GREEN: Metric.MetricColor.ValueType  # 1
    GRAY: Metric.MetricColor.ValueType  # 2
    ORANGE: Metric.MetricColor.ValueType  # 3
    YELLOW: Metric.MetricColor.ValueType  # 4
    BLUE: Metric.MetricColor.ValueType  # 5
    VIOLET: Metric.MetricColor.ValueType  # 6
    PRIMARY: Metric.MetricColor.ValueType  # 7

    class _ChartType:
        ValueType = _typing.NewType("ValueType", _builtins.int)
        V: _TypeAlias = ValueType  # noqa: Y015

    class _ChartTypeEnumTypeWrapper(_enum_type_wrapper._EnumTypeWrapper[Metric._ChartType.ValueType], _builtins.type):
        DESCRIPTOR: _descriptor.EnumDescriptor
        LINE: Metric._ChartType.ValueType  # 0
        BAR: Metric._ChartType.ValueType  # 1
        AREA: Metric._ChartType.ValueType  # 2

    class ChartType(_ChartType, metaclass=_ChartTypeEnumTypeWrapper): ...
    LINE: Metric.ChartType.ValueType  # 0
    BAR: Metric.ChartType.ValueType  # 1
    AREA: Metric.ChartType.ValueType  # 2

    LABEL_FIELD_NUMBER: _builtins.int
    BODY_FIELD_NUMBER: _builtins.int
    DELTA_FIELD_NUMBER: _builtins.int
    DIRECTION_FIELD_NUMBER: _builtins.int
    COLOR_FIELD_NUMBER: _builtins.int
    HELP_FIELD_NUMBER: _builtins.int
    LABEL_VISIBILITY_FIELD_NUMBER: _builtins.int
    SHOW_BORDER_FIELD_NUMBER: _builtins.int
    CHART_DATA_FIELD_NUMBER: _builtins.int
    CHART_TYPE_FIELD_NUMBER: _builtins.int
    FORMAT_FIELD_NUMBER: _builtins.int
    DELTA_DESCRIPTION_FIELD_NUMBER: _builtins.int
    label: _builtins.str
    body: _builtins.str
    delta: _builtins.str
    direction: Global___Metric.MetricDirection.ValueType
    color: Global___Metric.MetricColor.ValueType
    help: _builtins.str
    show_border: _builtins.bool
    chart_type: Global___Metric.ChartType.ValueType
    format: _builtins.str
    """Format string for the value and delta (applied only if they are numeric)."""
    delta_description: _builtins.str
    """Optional description text to display next to the delta value."""
    @_builtins.property
    def label_visibility(self) -> _LabelVisibility_pb2.LabelVisibility: ...
    @_builtins.property
    def chart_data(self) -> _containers.RepeatedScalarFieldContainer[_builtins.float]: ...
    def __init__(
        self,
        *,
        label: _builtins.str = ...,
        body: _builtins.str = ...,
        delta: _builtins.str = ...,
        direction: Global___Metric.MetricDirection.ValueType = ...,
        color: Global___Metric.MetricColor.ValueType = ...,
        help: _builtins.str = ...,
        label_visibility: _LabelVisibility_pb2.LabelVisibility | None = ...,
        show_border: _builtins.bool = ...,
        chart_data: _abc.Iterable[_builtins.float] | None = ...,
        chart_type: Global___Metric.ChartType.ValueType = ...,
        format: _builtins.str = ...,
        delta_description: _builtins.str = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["label_visibility", b"label_visibility"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["body", b"body", "chart_data", b"chart_data", "chart_type", b"chart_type", "color", b"color", "delta", b"delta", "delta_description", b"delta_description", "direction", b"direction", "format", b"format", "help", b"help", "label", b"label", "label_visibility", b"label_visibility", "show_border", b"show_border"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___Metric: _TypeAlias = Metric  # noqa: Y015



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*!
Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from streamlit.proto import ButtonLikeIconPosition_pb2 as _ButtonLikeIconPosition_pb2
import builtins as _builtins
import sys
import typing as _typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: _descriptor.FileDescriptor

@_typing.final
class LinkButton(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    ID_FIELD_NUMBER: _builtins.int
    LABEL_FIELD_NUMBER: _builtins.int
    HELP_FIELD_NUMBER: _builtins.int
    URL_FIELD_NUMBER: _builtins.int
    DISABLED_FIELD_NUMBER: _builtins.int
    USE_CONTAINER_WIDTH_FIELD_NUMBER: _builtins.int
    TYPE_FIELD_NUMBER: _builtins.int
    ICON_FIELD_NUMBER: _builtins.int
    SHORTCUT_FIELD_NUMBER: _builtins.int
    ICON_POSITION_FIELD_NUMBER: _builtins.int
    IGNORE_RERUN_FIELD_NUMBER: _builtins.int
    id: _builtins.str
    """The ID is needed for shortcuts, keys, and click-triggered reruns."""
    label: _builtins.str
    help: _builtins.str
    url: _builtins.str
    disabled: _builtins.bool
    use_container_width: _builtins.bool
    type: _builtins.str
    icon: _builtins.str
    shortcut: _builtins.str
    icon_position: _ButtonLikeIconPosition_pb2.ButtonLikeIconPosition.ValueType
    ignore_rerun: _builtins.bool
    def __init__(
        self,
        *,
        id: _builtins.str = ...,
        label: _builtins.str = ...,
        help: _builtins.str = ...,
        url: _builtins.str = ...,
        disabled: _builtins.bool = ...,
        use_container_width: _builtins.bool = ...,
        type: _builtins.str = ...,
        icon: _builtins.str = ...,
        shortcut: _builtins.str = ...,
        icon_position: _ButtonLikeIconPosition_pb2.ButtonLikeIconPosition.ValueType = ...,
        ignore_rerun: _builtins.bool = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["disabled", b"disabled", "help", b"help", "icon", b"icon", "icon_position", b"icon_position", "id", b"id", "ignore_rerun", b"ignore_rerun", "label", b"label", "shortcut", b"shortcut", "type", b"type", "url", b"url", "use_container_width", b"use_container_width"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___LinkButton: _TypeAlias = LinkButton  # noqa: Y015



# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Starlette integration for Streamlit.

This module provides the ASGI-compatible App class for running Streamlit
applications with any ASGI server (uvicorn, hypercorn, etc.).

Example
-------
>>> from streamlit.starlette import App
>>> app = App("main.py")

Run with uvicorn:

.. code-block:: bash

    uvicorn myapp:app --host 0.0.0.0 --port 8501
"""

from streamlit.web.server.starlette.starlette_app import App

__all__ = ["App"]



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*!
Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from streamlit.proto import ClientState_pb2 as _ClientState_pb2
from streamlit.proto import Common_pb2 as _Common_pb2
import builtins as _builtins
import sys
import typing as _typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: _descriptor.FileDescriptor

@_typing.final
class BackMsg(_message.Message):
    """A message from the browser to the server."""

    DESCRIPTOR: _descriptor.Descriptor

    CLEAR_CACHE_FIELD_NUMBER: _builtins.int
    SET_RUN_ON_SAVE_FIELD_NUMBER: _builtins.int
    STOP_SCRIPT_FIELD_NUMBER: _builtins.int
    RERUN_SCRIPT_FIELD_NUMBER: _builtins.int
    LOAD_GIT_INFO_FIELD_NUMBER: _builtins.int
    DEBUG_DISCONNECT_WEBSOCKET_FIELD_NUMBER: _builtins.int
    DEBUG_SHUTDOWN_RUNTIME_FIELD_NUMBER: _builtins.int
    FILE_URLS_REQUEST_FIELD_NUMBER: _builtins.int
    APP_HEARTBEAT_FIELD_NUMBER: _builtins.int
    DEFERRED_FILE_REQUEST_FIELD_NUMBER: _builtins.int
    DEBUG_LAST_BACKMSG_ID_FIELD_NUMBER: _builtins.int
    clear_cache: _builtins.bool
    """Requests that the app's @st_cache be cleared"""
    set_run_on_save: _builtins.bool
    """Requests that the runOnSave behavior for this app be set
    to the given value
    """
    stop_script: _builtins.bool
    """Requests that the script's execution be stopped"""
    load_git_info: _builtins.bool
    debug_disconnect_websocket: _builtins.bool
    """Test and dev-mode only field used to ask the server to disconnect the
    client's websocket connection. This message is IGNORED unless the
    runtime is configured with global.developmentMode = True.
    """
    debug_shutdown_runtime: _builtins.bool
    """Test and dev-mode only field used to ask the server to shut down the
    runtime. This message is IGNORED unless the runtime is configured with
    global.developmentMode = True.
    """
    app_heartbeat: _builtins.bool
    """Sends an app heartbeat message through the websocket"""
    debug_last_backmsg_id: _builtins.str
    """An ID used to associate this BackMsg with the corresponding ForwardMsgs
    that are sent to the client due to it. As its name suggests, this field
    should only be used for testing.
    """
    @_builtins.property
    def rerun_script(self) -> _ClientState_pb2.ClientState: ...
    @_builtins.property
    def file_urls_request(self) -> _Common_pb2.FileURLsRequest:
        """Requests that the server generate URLs for getting/uploading/deleting
        files for the `st.file_uploader` widget
        """

    @_builtins.property
    def deferred_file_request(self) -> Global___DeferredFileRequest:
        """Requests that the server execute a deferred callable and return the URL"""

    def __init__(
        self,
        *,
        clear_cache: _builtins.bool = ...,
        set_run_on_save: _builtins.bool = ...,
        stop_script: _builtins.bool = ...,
        rerun_script: _ClientState_pb2.ClientState | None = ...,
        load_git_info: _builtins.bool = ...,
        debug_disconnect_websocket: _builtins.bool = ...,
        debug_shutdown_runtime: _builtins.bool = ...,
        file_urls_request: _Common_pb2.FileURLsRequest | None = ...,
        app_heartbeat: _builtins.bool = ...,
        deferred_file_request: Global___DeferredFileRequest | None = ...,
        debug_last_backmsg_id: _builtins.str = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["app_heartbeat", b"app_heartbeat", "clear_cache", b"clear_cache", "debug_disconnect_websocket", b"debug_disconnect_websocket", "debug_shutdown_runtime", b"debug_shutdown_runtime", "deferred_file_request", b"deferred_file_request", "file_urls_request", b"file_urls_request", "load_git_info", b"load_git_info", "rerun_script", b"rerun_script", "set_run_on_save", b"set_run_on_save", "stop_script", b"stop_script", "type", b"type"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["app_heartbeat", b"app_heartbeat", "clear_cache", b"clear_cache", "debug_disconnect_websocket", b"debug_disconnect_websocket", "debug_last_backmsg_id", b"debug_last_backmsg_id", "debug_shutdown_runtime", b"debug_shutdown_runtime", "deferred_file_request", b"deferred_file_request", "file_urls_request", b"file_urls_request", "load_git_info", b"load_git_info", "rerun_script", b"rerun_script", "set_run_on_save", b"set_run_on_save", "stop_script", b"stop_script", "type", b"type"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType_type: _TypeAlias = _typing.Literal["clear_cache", "set_run_on_save", "stop_script", "rerun_script", "load_git_info", "debug_disconnect_websocket", "debug_shutdown_runtime", "file_urls_request", "app_heartbeat", "deferred_file_request"]  # noqa: Y015
    _WhichOneofArgType_type: _TypeAlias = _typing.Literal["type", b"type"]  # noqa: Y015
    def WhichOneof(self, oneof_group: _WhichOneofArgType_type) -> _WhichOneofReturnType_type | None: ...

Global___BackMsg: _TypeAlias = BackMsg  # noqa: Y015

@_typing.final
class DeferredFileRequest(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    FILE_ID_FIELD_NUMBER: _builtins.int
    SESSION_ID_FIELD_NUMBER: _builtins.int
    file_id: _builtins.str
    session_id: _builtins.str
    def __init__(
        self,
        *,
        file_id: _builtins.str = ...,
        session_id: _builtins.str = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["file_id", b"file_id", "session_id", b"session_id"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___DeferredFileRequest: _TypeAlias = DeferredFileRequest  # noqa: Y015



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*!
Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from collections import abc as _abc
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf.internal import containers as _containers
from streamlit.proto import WidgetStates_pb2 as _WidgetStates_pb2
import builtins as _builtins
import sys
import typing as _typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: _descriptor.FileDescriptor

@_typing.final
class ContextInfo(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    TIMEZONE_FIELD_NUMBER: _builtins.int
    TIMEZONE_OFFSET_FIELD_NUMBER: _builtins.int
    LOCALE_FIELD_NUMBER: _builtins.int
    URL_FIELD_NUMBER: _builtins.int
    IS_EMBEDDED_FIELD_NUMBER: _builtins.int
    COLOR_SCHEME_FIELD_NUMBER: _builtins.int
    timezone: _builtins.str
    timezone_offset: _builtins.int
    locale: _builtins.str
    url: _builtins.str
    is_embedded: _builtins.bool
    color_scheme: _builtins.str
    def __init__(
        self,
        *,
        timezone: _builtins.str | None = ...,
        timezone_offset: _builtins.int | None = ...,
        locale: _builtins.str | None = ...,
        url: _builtins.str | None = ...,
        is_embedded: _builtins.bool | None = ...,
        color_scheme: _builtins.str | None = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["_color_scheme", b"_color_scheme", "_is_embedded", b"_is_embedded", "_locale", b"_locale", "_timezone", b"_timezone", "_timezone_offset", b"_timezone_offset", "_url", b"_url", "color_scheme", b"color_scheme", "is_embedded", b"is_embedded", "locale", b"locale", "timezone", b"timezone", "timezone_offset", b"timezone_offset", "url", b"url"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["_color_scheme", b"_color_scheme", "_is_embedded", b"_is_embedded", "_locale", b"_locale", "_timezone", b"_timezone", "_timezone_offset", b"_timezone_offset", "_url", b"_url", "color_scheme", b"color_scheme", "is_embedded", b"is_embedded", "locale", b"locale", "timezone", b"timezone", "timezone_offset", b"timezone_offset", "url", b"url"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType__color_scheme: _TypeAlias = _typing.Literal["color_scheme"]  # noqa: Y015
    _WhichOneofArgType__color_scheme: _TypeAlias = _typing.Literal["_color_scheme", b"_color_scheme"]  # noqa: Y015
    _WhichOneofReturnType__is_embedded: _TypeAlias = _typing.Literal["is_embedded"]  # noqa: Y015
    _WhichOneofArgType__is_embedded: _TypeAlias = _typing.Literal["_is_embedded", b"_is_embedded"]  # noqa: Y015
    _WhichOneofReturnType__locale: _TypeAlias = _typing.Literal["locale"]  # noqa: Y015
    _WhichOneofArgType__locale: _TypeAlias = _typing.Literal["_locale", b"_locale"]  # noqa: Y015
    _WhichOneofReturnType__timezone: _TypeAlias = _typing.Literal["timezone"]  # noqa: Y015
    _WhichOneofArgType__timezone: _TypeAlias = _typing.Literal["_timezone", b"_timezone"]  # noqa: Y015
    _WhichOneofReturnType__timezone_offset: _TypeAlias = _typing.Literal["timezone_offset"]  # noqa: Y015
    _WhichOneofArgType__timezone_offset: _TypeAlias = _typing.Literal["_timezone_offset", b"_timezone_offset"]  # noqa: Y015
    _WhichOneofReturnType__url: _TypeAlias = _typing.Literal["url"]  # noqa: Y015
    _WhichOneofArgType__url: _TypeAlias = _typing.Literal["_url", b"_url"]  # noqa: Y015
    @_typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__color_scheme) -> _WhichOneofReturnType__color_scheme | None: ...
    @_typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__is_embedded) -> _WhichOneofReturnType__is_embedded | None: ...
    @_typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__locale) -> _WhichOneofReturnType__locale | None: ...
    @_typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__timezone) -> _WhichOneofReturnType__timezone | None: ...
    @_typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__timezone_offset) -> _WhichOneofReturnType__timezone_offset | None: ...
    @_typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__url) -> _WhichOneofReturnType__url | None: ...

Global___ContextInfo: _TypeAlias = ContextInfo  # noqa: Y015

@_typing.final
class ClientState(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    QUERY_STRING_FIELD_NUMBER: _builtins.int
    WIDGET_STATES_FIELD_NUMBER: _builtins.int
    PAGE_SCRIPT_HASH_FIELD_NUMBER: _builtins.int
    PAGE_NAME_FIELD_NUMBER: _builtins.int
    FRAGMENT_ID_FIELD_NUMBER: _builtins.int
    IS_AUTO_RERUN_FIELD_NUMBER: _builtins.int
    CACHED_MESSAGE_HASHES_FIELD_NUMBER: _builtins.int
    CONTEXT_INFO_FIELD_NUMBER: _builtins.int
    query_string: _builtins.str
    page_script_hash: _builtins.str
    page_name: _builtins.str
    fragment_id: _builtins.str
    is_auto_rerun: _builtins.bool
    @_builtins.property
    def widget_states(self) -> _WidgetStates_pb2.WidgetStates: ...
    @_builtins.property
    def cached_message_hashes(self) -> _containers.RepeatedScalarFieldContainer[_builtins.str]:
        """List of hashes of messages that are currently cached in
        the frontend forward message cache.
        """

    @_builtins.property
    def context_info(self) -> Global___ContextInfo: ...
    def __init__(
        self,
        *,
        query_string: _builtins.str = ...,
        widget_states: _WidgetStates_pb2.WidgetStates | None = ...,
        page_script_hash: _builtins.str = ...,
        page_name: _builtins.str = ...,
        fragment_id: _builtins.str = ...,
        is_auto_rerun: _builtins.bool = ...,
        cached_message_hashes: _abc.Iterable[_builtins.str] | None = ...,
        context_info: Global___ContextInfo | None = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["context_info", b"context_info", "widget_states", b"widget_states"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["cached_message_hashes", b"cached_message_hashes", "context_info", b"context_info", "fragment_id", b"fragment_id", "is_auto_rerun", b"is_auto_rerun", "page_name", b"page_name", "page_script_hash", b"page_script_hash", "query_string", b"query_string", "widget_states", b"widget_states"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___ClientState: _TypeAlias = ClientState  # noqa: Y015



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*!
Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from collections import abc as _abc
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf.internal import containers as _containers
import builtins as _builtins
import sys
import typing as _typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: _descriptor.FileDescriptor

@_typing.final
class Image(_message.Message):
    """An image which can be displayed on the screen."""

    DESCRIPTOR: _descriptor.Descriptor

    URL_FIELD_NUMBER: _builtins.int
    CAPTION_FIELD_NUMBER: _builtins.int
    url: _builtins.str
    caption: _builtins.str
    def __init__(
        self,
        *,
        url: _builtins.str = ...,
        caption: _builtins.str = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["caption", b"caption", "url", b"url"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___Image: _TypeAlias = Image  # noqa: Y015

@_typing.final
class ImageList(_message.Message):
    """A set of images."""

    DESCRIPTOR: _descriptor.Descriptor

    IMGS_FIELD_NUMBER: _builtins.int
    LINK_FIELD_NUMBER: _builtins.int
    link: _builtins.str
    """External URL to open when the image is clicked.
    Only supported when there is exactly one image in the list.
    """
    @_builtins.property
    def imgs(self) -> _containers.RepeatedCompositeFieldContainer[Global___Image]: ...
    def __init__(
        self,
        *,
        imgs: _abc.Iterable[Global___Image] | None = ...,
        link: _builtins.str = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["imgs", b"imgs", "link", b"link"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___ImageList: _TypeAlias = ImageList  # noqa: Y015



# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import annotations

from typing import TYPE_CHECKING, Final, cast

import tornado.web

import streamlit.web.server.routes
from streamlit.logger import get_logger
from streamlit.web.server.component_file_utils import (
    build_safe_abspath,
    guess_content_type,
)

if TYPE_CHECKING:
    from streamlit.components.types.base_component_registry import BaseComponentRegistry

_LOGGER: Final = get_logger(__name__)


class ComponentRequestHandler(tornado.web.RequestHandler):
    def initialize(self, registry: BaseComponentRegistry) -> None:
        self._registry = registry

    def get(self, path: str) -> None:
        parts = path.split("/")
        component_name = parts[0]
        component_root = self._registry.get_component_path(component_name)
        if component_root is None:
            self.write("not found")
            self.set_status(404)
            return

        # Build a safe absolute path within the component root
        filename = "/".join(parts[1:])
        abspath = build_safe_abspath(component_root, filename)
        if abspath is None:
            self.write("Bad Request")
            self.set_status(400)
            return
        try:
            with open(abspath, "rb") as file:
                contents = file.read()
        except OSError:
            sanitized_abspath = abspath.replace("\n", "").replace("\r", "")
            _LOGGER.exception(
                "ComponentRequestHandler: GET %s read error", sanitized_abspath
            )
            self.write("read error")
            self.set_status(404)
            return

        self.write(contents)
        self.set_header("Content-Type", self.get_content_type(abspath))

        self.set_extra_headers(path)

    def set_extra_headers(self, path: str) -> None:
        """Disable cache for HTML files.

        Other assets like JS and CSS are suffixed with their hash, so they can
        be cached indefinitely.
        """
        is_index_url = len(path) == 0

        if is_index_url or path.endswith(".html"):
            self.set_header("Cache-Control", "no-cache")
        else:
            self.set_header("Cache-Control", "public")

    def set_default_headers(self) -> None:
        if streamlit.web.server.routes.allow_all_cross_origin_requests():
            self.set_header("Access-Control-Allow-Origin", "*")
        elif streamlit.web.server.routes.is_allowed_origin(
            origin := self.request.headers.get("Origin")
        ):
            self.set_header("Access-Control-Allow-Origin", cast("str", origin))

    def options(self) -> None:
        """/OPTIONS handler for preflight CORS checks."""
        self.set_status(204)
        self.finish()

    @staticmethod
    def get_content_type(abspath: str) -> str:
        """Returns the ``Content-Type`` header to be used for this request.
        From tornado.web.StaticFileHandler.
        """
        return guess_content_type(abspath)

    @staticmethod
    def get_url(file_id: str) -> str:
        """Return the URL for a component file with the given ID."""
        return f"components/{file_id}"



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*!
Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from collections import abc as _abc
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
import builtins as _builtins
import sys
import typing as _typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: _descriptor.FileDescriptor

@_typing.final
class DeckGlJsonChart(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    class _SelectionMode:
        ValueType = _typing.NewType("ValueType", _builtins.int)
        V: _TypeAlias = ValueType  # noqa: Y015

    class _SelectionModeEnumTypeWrapper(_enum_type_wrapper._EnumTypeWrapper[DeckGlJsonChart._SelectionMode.ValueType], _builtins.type):
        DESCRIPTOR: _descriptor.EnumDescriptor
        SINGLE_OBJECT: DeckGlJsonChart._SelectionMode.ValueType  # 0
        """Only one object can be selected at a time."""
        MULTI_OBJECT: DeckGlJsonChart._SelectionMode.ValueType  # 1
        """Multiple objects can be selected at a time."""

    class SelectionMode(_SelectionMode, metaclass=_SelectionModeEnumTypeWrapper):
        """Available selection modes:"""

    SINGLE_OBJECT: DeckGlJsonChart.SelectionMode.ValueType  # 0
    """Only one object can be selected at a time."""
    MULTI_OBJECT: DeckGlJsonChart.SelectionMode.ValueType  # 1
    """Multiple objects can be selected at a time."""

    JSON_FIELD_NUMBER: _builtins.int
    TOOLTIP_FIELD_NUMBER: _builtins.int
    ID_FIELD_NUMBER: _builtins.int
    MAPBOX_TOKEN_FIELD_NUMBER: _builtins.int
    SELECTION_MODE_FIELD_NUMBER: _builtins.int
    FORM_ID_FIELD_NUMBER: _builtins.int
    json: _builtins.str
    """The json of the pydeck object (https://deckgl.readthedocs.io/en/latest/deck.html)"""
    tooltip: _builtins.str
    id: _builtins.str
    """ID, required for selection events."""
    mapbox_token: _builtins.str
    """The Mapbox token, if any."""
    form_id: _builtins.str
    """The form ID of the widget, this is required if the chart has selection events"""
    @_builtins.property
    def selection_mode(self) -> _containers.RepeatedScalarFieldContainer[Global___DeckGlJsonChart.SelectionMode.ValueType]:
        """If non-empty, treat this instance as a Widget"""

    def __init__(
        self,
        *,
        json: _builtins.str = ...,
        tooltip: _builtins.str = ...,
        id: _builtins.str = ...,
        mapbox_token: _builtins.str = ...,
        selection_mode: _abc.Iterable[Global___DeckGlJsonChart.SelectionMode.ValueType] | None = ...,
        form_id: _builtins.str = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["form_id", b"form_id", "id", b"id", "json", b"json", "mapbox_token", b"mapbox_token", "selection_mode", b"selection_mode", "tooltip", b"tooltip"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___DeckGlJsonChart: _TypeAlias = DeckGlJsonChart  # noqa: Y015
